"""
MySQL数据库客户端模块

提供MySQL数据库操作的封装，支持：
- 连接池管理
- 线程安全操作
- 批量插入/替换/删除
- 分页查询
- 自动重连机制
- 参数化查询（防SQL注入）

依赖：
    pip install pymysql DBUtils mysql-connector-python
"""
import logging
import threading
from typing import Optional, List, Dict, Any, Type, Tuple, Union

import pymysql
import mysql.connector
import mysql.connector.errorcode
from dbutils.pooled_db import PooledDB

from xsdk.xutil import Xutil
from xsdk.bean_util import BaseBean, BeanUtil
from xsdk.config_reader import ConfigReader

logger = logging.getLogger(__name__)

# 参数类型定义
Params = Optional[Union[Tuple, List, Dict]]


class DBError(Exception):
    """
    数据库操作异常类

    Attributes:
        code: 错误码
        message: 错误消息
    """

    def __init__(self, code: int = -100, message: str = "DB_ERROR"):
        super().__init__(message)
        self.code = code
        self.message = message

    def __str__(self) -> str:
        return str({"code": self.code, "message": self.message})


class DBResult:
    """
    数据库操作结果类

    Attributes:
        code: 状态码（0成功，-1失败）
        message: 状态消息
        data: 查询结果数据
        row_count: 影响行数
        page_info: 分页信息
    """

    CODE_SUCCESS = 0
    CODE_FAILED = -1
    MESSAGE_SUCCESS = "SUCCESS"
    MESSAGE_FAILED = "FAILED"

    def __init__(
        self,
        code: int = CODE_SUCCESS,
        message: str = MESSAGE_SUCCESS,
        data: Optional[List[Dict]] = None,
        row_count: int = 0
    ):
        self.code = code
        self.message = message
        self.data = data if data is not None else []
        self.row_count = row_count
        self.page_info: Optional["PageUtil"] = None

    def __str__(self) -> str:
        return str(self.__dict__)

    def is_failed(self) -> bool:
        """检查操作是否失败"""
        return self.code == DBResult.CODE_FAILED

    def is_success(self) -> bool:
        """检查操作是否成功"""
        return self.code == DBResult.CODE_SUCCESS

    @staticmethod
    def fail(msg: Optional[str] = None) -> "DBResult":
        """创建失败结果"""
        if Xutil.is_empty(msg):
            msg = DBResult.MESSAGE_FAILED
        return DBResult(DBResult.CODE_FAILED, msg)

    @staticmethod
    def success() -> "DBResult":
        """创建成功结果"""
        return DBResult(DBResult.CODE_SUCCESS, DBResult.MESSAGE_SUCCESS)


class PageUtil:
    """
    分页工具类

    Attributes:
        total_pages: 总页数
        total_records: 总记录数
        page_number: 当前页码（从1开始）
        page_size: 每页记录数
        has_prev: 是否有上一页
        has_next: 是否有下一页
        prev_page: 上一页页码
        next_page: 下一页页码
    """

    def __init__(self, total_records: int, page_number: int = 1, page_size: int = 20):
        self.total_pages = 0
        self.total_records = total_records
        self.page_number = page_number
        self.page_size = page_size
        self.has_prev = False
        self.has_next = False
        self.prev_page = 0
        self.next_page = 0
        self._reset()

    def __str__(self) -> str:
        return str(self.__dict__)

    def _reset(self) -> None:
        """重新计算分页信息"""
        # 参数校验
        if self.total_records is None or int(self.total_records) < 0:
            self.total_records = 0
        if self.page_number is None or int(self.page_number) < 1:
            self.page_number = 1
        if self.page_size is None or int(self.page_size) <= 0:
            self.page_size = 20

        self.total_records = int(self.total_records)
        self.page_number = int(self.page_number)
        self.page_size = int(self.page_size)

        # 计算总页数
        if self.total_records % self.page_size == 0:
            self.total_pages = self.total_records // self.page_size
        else:
            self.total_pages = self.total_records // self.page_size + 1

        # 计算上一页
        if self.page_number > 1:
            self.has_prev = True
            self.prev_page = self.page_number - 1
        else:
            self.has_prev = False
            self.prev_page = self.page_number

        # 计算下一页
        if self.page_number < self.total_pages:
            self.has_next = True
            self.next_page = self.page_number + 1
        else:
            self.has_next = False
            self.next_page = self.page_number

    @staticmethod
    def get_total(db_instance: "MySQLClient", sql: str, params: Params = None) -> int:
        """
        获取SQL查询的总记录数

        Args:
            db_instance: MySQLClient实例
            sql: 原始查询SQL
            params: SQL参数（用于参数化查询）

        Returns:
            总记录数
        """
        sql_total = f"SELECT COUNT(*) AS total FROM ({sql}) t"
        db_ret = db_instance.query(sql_total, params=params)

        total = 0
        if db_ret and len(db_ret.data) == 1:
            total = int(db_ret.data[0]["total"])
        return total


class MySQLConfig:
    """
    MySQL连接配置类

    Attributes:
        host: 数据库主机
        port: 数据库端口
        user: 用户名
        password: 密码
        timeout: 连接超时时间
        charset: 字符集
    """

    def __init__(
        self,
        host: str = "127.0.0.1",
        port: int = 3306,
        user: str = "root",
        password: str = "",
        timeout: int = 5000,
        charset: str = "utf8"
    ):
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.timeout = timeout
        self.charset = charset
        self.use_unicode = True
        self.use_pure = False

    @staticmethod
    def get_config(conf_file: str, conf_section: str) -> "MySQLConfig":
        """
        从配置文件读取MySQL配置

        Args:
            conf_file: 配置文件路径
            conf_section: 配置节名称

        Returns:
            MySQLConfig实例
        """
        config_reader = ConfigReader.get_instance(conf_file)
        host = config_reader.get(conf_section, "host")
        port = config_reader.get_int(conf_section, "port")
        user = config_reader.get(conf_section, "user")
        password = config_reader.get(conf_section, "password")
        timeout = config_reader.get_int(conf_section, "timeout")
        charset = config_reader.get(conf_section, "charset") or "utf8"

        return MySQLConfig(host, port, user, password, timeout, charset)

    @staticmethod
    def parse_dict(data_dict: Dict[str, Any]) -> "MySQLConfig":
        """
        从字典解析MySQL配置

        Args:
            data_dict: 配置字典

        Returns:
            MySQLConfig实例
        """
        db_config = MySQLConfig()
        db_config.__dict__ = data_dict
        return db_config


class MySQLClient:
    """
    MySQL客户端封装类

    提供线程安全的数据库操作，支持连接池、自动重连和参数化查询。

    支持两种调用方式：
        # 方式1 - 传统字符串拼接（兼容旧代码）
        >>> result = client.query(f"SELECT * FROM users WHERE id = '{user_id}'")

        # 方式2 - 参数化查询（推荐，防SQL注入）
        >>> result = client.query("SELECT * FROM users WHERE id = %s", params=(user_id,))
    """

    def __init__(self, mysql_config: MySQLConfig, logger: logging.Logger = logger):
        self.logger = logger
        self.host = mysql_config.host
        self.port = mysql_config.port
        self.user = mysql_config.user
        self.password = mysql_config.password
        self.timeout = mysql_config.timeout
        self.charset = mysql_config.charset
        self.use_unicode = True
        self.use_pure = None
        self.pool: Optional[PooledDB] = None
        self.local_thread_data = threading.local()
        self._init()

    def __str__(self) -> str:
        return str(self.__dict__)

    def _init(self) -> None:
        """初始化连接池（仅创建池，不预取连接，避免主线程占用连接槽位）"""
        try:
            min_cached = 4
            max_cached = 8
            max_shared = 0
            max_connections = 64
            max_usage = 8
            set_session = None
            blocking = True
            reset = True

            self.pool = PooledDB(
                pymysql,
                min_cached,
                max_cached,
                max_shared,
                max_connections,
                blocking,
                max_usage,
                set_session,
                reset,
                host=self.host,
                port=self.port,
                user=self.user,
                passwd=self.password,
                charset=self.charset,
                cursorclass=pymysql.cursors.DictCursor
            )
        except Exception as error:
            raise DBError(message=str(error))

    def _ensure_connection(self) -> None:
        """
        确保当前线程持有可用的数据库连接和游标

        处理三种场景：
        1. 当前线程首次访问，无连接 → 从连接池获取新连接
        2. 已有连接，ping 存活检测通过 → 复用现有连接
        3. 已有连接，但 ping 失败（连接已断开）→ 关闭旧连接，从连接池获取新连接
        """
        thread_id = str(threading.current_thread().ident)
        self.local_thread_data.thread_id = thread_id

        try:
            conn = getattr(self.local_thread_data, "connection", None)

            if conn is None:
                # 场景1：当前线程无连接，从连接池获取
                self.local_thread_data.connection = self.pool.connection()
                self.local_thread_data.cursor = self.local_thread_data.connection.cursor()
                self.logger.info(
                    f"reconnect.001: new connection | thread={thread_id} | "
                    f"conn={hex(id(self.local_thread_data.connection))}",
                    stacklevel=3
                )
                return

            # 场景2/3：已有连接，检测存活
            try:
                conn.ping(reconnect=True)
                self.logger.info(
                    f"reconnect.002: ping ok | thread={thread_id} | "
                    f"conn={hex(id(conn))}",
                    stacklevel=3
                )
            except Exception:
                # 场景3：ping 失败，关闭旧连接后重新获取
                self.logger.warning(
                    f"reconnect.003: ping failed, replacing connection | thread={thread_id}",
                    stacklevel=3
                )
                try:
                    conn.close()
                except Exception:
                    pass
                self.local_thread_data.connection = self.pool.connection()

            # 每次都创建新 cursor（cursor 是轻量对象，确保状态干净）
            self.local_thread_data.cursor = self.local_thread_data.connection.cursor()

        except Exception as error:
            self.logger.error(f"Failed to get db connection | thread={thread_id}", stacklevel=3)
            self.logger.exception(error, stacklevel=3)
            raise error

    def reconnect(self) -> None:
        """重新建立数据库连接（兼容旧接口，内部委托给 _ensure_connection）"""
        self._ensure_connection()

    def get_connection(self):
        """获取数据库连接"""
        self.reconnect()
        return self.local_thread_data.connection

    @staticmethod
    def get_instance(
        conf_file: str,
        conf_section: str = "db",
        logger: logging.Logger = logger
    ) -> "MySQLClient":
        """
        从配置文件创建MySQLClient实例

        Args:
            conf_file: 配置文件路径
            conf_section: 配置节名称
            logger: 日志记录器

        Returns:
            MySQLClient实例
        """
        db_config = MySQLConfig.get_config(conf_file, conf_section)
        return MySQLClient(db_config, logger)

    @staticmethod
    def _is_connection_error(error: Exception) -> bool:
        """判断是否为连接断开类异常（可通过重连恢复）"""
        if isinstance(error, pymysql.err.OperationalError):
            # CR_SERVER_GONE_ERROR(2006), CR_SERVER_LOST(2013)
            err_code = error.args[0] if error.args else 0
            return err_code in (2006, 2013)
        if isinstance(error, pymysql.err.InterfaceError):
            return True
        err_msg = str(error).lower()
        return any(keyword in err_msg for keyword in (
            "server has gone away",
            "lost connection",
            "broken pipe",
            "connection reset",
            "not connected",
        ))

    def _force_new_connection(self) -> None:
        """强制丢弃当前连接，从连接池获取新连接和游标"""
        thread_id = getattr(self.local_thread_data, "thread_id", "unknown")
        old_conn = getattr(self.local_thread_data, "connection", None)
        if old_conn is not None:
            try:
                old_conn.close()
            except Exception:
                pass
        self.local_thread_data.connection = self.pool.connection()
        self.local_thread_data.cursor = self.local_thread_data.connection.cursor()
        self.logger.warning(
            f"reconnect.retry: forced new connection | thread={thread_id} | "
            f"conn={hex(id(self.local_thread_data.connection))}",
            stacklevel=4
        )

    def _execute(
        self,
        sql: str,
        params: Params = None,
        need_fetch: bool = False,
        auto_commit: bool = False
    ) -> DBResult:
        """
        执行SQL语句（内部方法）

        连接断开时自动重连重试一次。

        Args:
            sql: SQL语句（可包含 %s 占位符）
            params: SQL参数，与 %s 占位符对应
            need_fetch: 是否需要获取结果
            auto_commit: 是否自动提交

        Returns:
            DBResult实例
        """
        self.local_thread_data.db_ret = DBResult()
        last_error = None

        for attempt in range(2):  # 最多重试 1 次
            try:
                if attempt == 0:
                    self._ensure_connection()
                else:
                    # 第二次尝试：强制获取新连接
                    self.logger.warning(
                        f"Retrying after connection error: {last_error}",
                        stacklevel=3
                    )
                    self._force_new_connection()

                if params is not None:
                    self.local_thread_data.cursor.execute(sql, params)
                else:
                    self.local_thread_data.cursor.execute(sql)
                if auto_commit:
                    self.local_thread_data.connection.commit()
                if need_fetch:
                    self.local_thread_data.db_ret = DBResult(data=self.local_thread_data.cursor.fetchall())
                self.local_thread_data.db_ret.row_count = self.local_thread_data.cursor.rowcount
                self.logger.info(f"Affected row count: {self.local_thread_data.db_ret.row_count}", stacklevel=3)
                return self.local_thread_data.db_ret

            except Exception as error:
                # 连接断开类错误且是首次尝试 → 重试
                if attempt == 0 and self._is_connection_error(error):
                    last_error = error
                    continue
                # 非连接错误或重试仍失败 → rollback 后抛出
                try:
                    self.local_thread_data.connection.rollback()
                except Exception:
                    pass
                raise DBError(message=str(error))

        # 理论上不会走到这里
        raise DBError(message=str(last_error))

    def _execute_many(
        self,
        sql: str,
        params_list: List[Tuple],
        auto_commit: bool = True
    ) -> DBResult:
        """
        批量执行参数化SQL（内部方法，用于 batch_insert / batch_replace）

        连接断开时自动重连重试一次。
        PyMySQL 的 executemany 会将 INSERT/REPLACE 自动优化为多行 VALUES 语法。

        Args:
            sql: SQL模板，如 INSERT INTO t(a,b) VALUES (%s,%s)
            params_list: 参数元组列表
            auto_commit: 是否自动提交

        Returns:
            DBResult实例
        """
        self.local_thread_data.db_ret = DBResult()
        last_error = None

        for attempt in range(2):
            try:
                if attempt == 0:
                    self._ensure_connection()
                else:
                    self.logger.warning(
                        f"Retrying executemany after connection error: {last_error}",
                        stacklevel=3
                    )
                    self._force_new_connection()

                self.local_thread_data.cursor.executemany(sql, params_list)
                if auto_commit:
                    self.local_thread_data.connection.commit()
                self.local_thread_data.db_ret.row_count = self.local_thread_data.cursor.rowcount
                self.logger.info(f"Affected row count: {self.local_thread_data.db_ret.row_count}", stacklevel=3)
                return self.local_thread_data.db_ret

            except Exception as error:
                if attempt == 0 and self._is_connection_error(error):
                    last_error = error
                    continue
                try:
                    self.local_thread_data.connection.rollback()
                except Exception:
                    pass
                raise DBError(message=str(error))

        raise DBError(message=str(last_error))

    def query(
        self,
        sql: str,
        params: Params = None,
        cls: Optional[Type[BaseBean]] = None,
        auto_commit: bool = False
    ) -> DBResult:
        """
        执行查询SQL

        Args:
            sql: SELECT语句（可包含 %s 占位符）
            params: SQL参数，与 %s 占位符对应
            cls: 结果映射的Bean类（需继承BaseBean）
            auto_commit: 是否自动提交

        Returns:
            DBResult实例

        Raises:
            DBError: 非SELECT语句时抛出

        Example:
            # 传统方式（兼容）
            >>> db.query(f"SELECT * FROM users WHERE id = '{uid}'")

            # 参数化查询（推荐）
            >>> db.query("SELECT * FROM users WHERE id = %s", params=(uid,))
            >>> db.query("SELECT * FROM users WHERE name = %s AND age > %s", params=(name, 18), cls=User)
        """
        log_msg = Xutil.simple_str(sql)
        if params is not None:
            log_msg += f" | params: {str(params)[:512]}"
        self.logger.info(log_msg, stacklevel=3)

        # SQL 校验
        if not sql.strip().lower().startswith(("select", "desc", "show")):
            raise DBError(message=f"not select|desc|show sql! {sql}")

        # 中文编码处理（仅对无参数的拼接SQL生效）
        if params is None and Xutil.contains_chinese(sql) and self.charset.lower() == "latin1":
            sql = sql.encode("gbk", "ignore")

        db_ret = self._execute(sql, params=params, need_fetch=True, auto_commit=auto_commit)

        # 结果映射到Bean
        if cls and issubclass(cls, BaseBean):
            if db_ret and db_ret.data:
                bean_list = BeanUtil.parse2bean_list(db_ret.data, cls)
                db_ret.data = bean_list

        return db_ret

    def query_page(
        self,
        sql: str,
        page_number: int,
        page_size: int,
        params: Params = None,
        cls: Optional[Type[BaseBean]] = None
    ) -> Optional[DBResult]:
        """
        分页查询

        Args:
            sql: SELECT语句（可包含 %s 占位符）
            page_number: 页码（从1开始）
            page_size: 每页记录数
            params: SQL参数，与 %s 占位符对应
            cls: 结果映射的Bean类

        Returns:
            包含分页信息的DBResult，无数据时返回None

        Example:
            # 传统方式
            >>> db.query_page(f"SELECT * FROM users WHERE age > {min_age}", 1, 20)

            # 参数化查询
            >>> db.query_page("SELECT * FROM users WHERE age > %s", 1, 20, params=(min_age,))
        """
        sql = str(sql).strip()
        if sql.endswith(";"):
            sql = sql[:-1]

        total_records = PageUtil.get_total(self, sql, params=params)
        if total_records == 0:
            return None

        page_info = PageUtil(total_records, page_number, page_size)
        idx_limit = (page_info.page_number - 1) * page_info.page_size
        sql_page = f"{sql} LIMIT {idx_limit}, {page_info.page_size}"

        db_ret = self.query(sql_page, params=params, cls=cls)
        db_ret.page_info = page_info
        return db_ret

    def execute(self, sql: str, params: Params = None, auto_commit: bool = True) -> DBResult:
        """
        执行SQL语句

        Args:
            sql: SQL语句（可包含 %s 占位符）
            params: SQL参数，与 %s 占位符对应
            auto_commit: 是否自动提交

        Returns:
            DBResult实例

        Example:
            # 传统方式
            >>> db.execute(f"DELETE FROM users WHERE id = '{uid}'")

            # 参数化查询
            >>> db.execute("DELETE FROM users WHERE id = %s", params=(uid,))
        """
        simple_sql = Xutil.simple_str(sql)
        if len(simple_sql) > 8192:
            simple_sql = f"{simple_sql[:8192]} ..."
        if params is not None:
            simple_sql += f" | params: {str(params)[:512]}"
        self.logger.info(simple_sql, stacklevel=3)

        # 中文编码处理（仅对无参数的拼接SQL生效）
        if params is None and Xutil.contains_chinese(sql) and self.charset.lower() == "latin1":
            sql = sql.encode("gbk", "ignore")

        return self._execute(sql, params=params, need_fetch=True, auto_commit=auto_commit)

    def update(self, sql: str, params: Params = None, auto_commit: bool = True) -> DBResult:
        """
        执行更新SQL

        Args:
            sql: 更新SQL语句（可包含 %s 占位符）
            params: SQL参数，与 %s 占位符对应
            auto_commit: 是否自动提交

        Returns:
            DBResult实例

        Example:
            # 传统方式
            >>> db.update(f"UPDATE users SET name = '{name}' WHERE id = '{uid}'")

            # 参数化查询
            >>> db.update("UPDATE users SET name = %s WHERE id = %s", params=(name, uid))
        """
        simple_sql = Xutil.simple_str(sql)
        if len(simple_sql) > 8192:
            simple_sql = f"{simple_sql[:8192]} ..."
        if params is not None:
            simple_sql += f" | params: {str(params)[:512]}"
        self.logger.info(simple_sql, stacklevel=3)

        # SQL 校验
        _ALLOWED_UPDATE_PREFIXES = ("delete", "insert", "replace", "truncate", "update", "alter", "set", "create")
        sql_lower = sql.strip().lower()
        if not sql_lower.startswith(_ALLOWED_UPDATE_PREFIXES):
            raise DBError(message=f"not delete|insert|update|replace|truncate|alter|set|create sql! {sql}")

        # 中文编码处理（仅对无参数的拼接SQL生效）
        if params is None and Xutil.contains_chinese(sql) and self.charset.lower() == "latin1":
            sql = sql.encode("gbk", "ignore")

        return self._execute(sql, params=params, need_fetch=True, auto_commit=auto_commit)

    def update_object(
        self,
        data_dict: Dict[str, Any],
        db_name: str,
        tb_name: str,
        key_fields: List[str],
        auto_commit: bool = True
    ) -> None:
        """
        更新对象数据（内部使用参数化查询）

        Args:
            data_dict: 数据字典或对象
            db_name: 数据库名
            tb_name: 表名
            key_fields: 主键字段列表（只用于定位，不更新）
            auto_commit: 是否自动提交
        """
        if not key_fields:
            self.logger.info("No key fields specified", stacklevel=3)
            return
        if not data_dict:
            self.logger.info("No data to update", stacklevel=3)
            return

        if not isinstance(data_dict, dict):
            data_dict = data_dict.__dict__

        # 构建SET子句（参数化）
        set_parts = []
        set_values = []
        for key, value in data_dict.items():
            if key not in key_fields:
                set_parts.append(f"{key} = %s")
                set_values.append(value)

        # 构建WHERE子句（参数化）
        where_parts = ["1=1"]
        where_values = []
        for key, value in data_dict.items():
            if key in key_fields:
                where_parts.append(f"{key} = %s")
                where_values.append(value)

        sql = f"UPDATE {db_name}.{tb_name} SET {', '.join(set_parts)} WHERE {' AND '.join(where_parts)}"
        params = tuple(set_values + where_values)
        self.update(sql, params=params, auto_commit=auto_commit)

    def batch_insert(
        self,
        data_list: List[Any],
        db_name: str,
        tb_name: str,
        batch_size: int = 3000,
        auto_keys: List[str] = None
    ) -> None:
        """
        批量插入数据（内部使用参数化查询）

        Args:
            data_list: 字典列表或对象列表
            db_name: 数据库名
            tb_name: 表名
            batch_size: 每批插入的记录数
            auto_keys: 自增字段列表，默认["id", "fid"]
        """
        if auto_keys is None:
            auto_keys = ["id", "fid"]

        if not data_list:
            self.logger.info(f"[batch_insert] No data to insert: {db_name}.{tb_name}", stacklevel=3)
            return

        self.logger.info(f"[batch_insert] Data size: {len(data_list)}", stacklevel=3)

        # 获取字段列表
        first = data_list[0]
        if not isinstance(first, dict):
            first = first.__dict__

        field_list = [key for key in first.keys() if key.lower() not in auto_keys]
        field_str = ", ".join(field_list)
        placeholders = ", ".join(["%s"] * len(field_list))
        sql = f"INSERT INTO {db_name}.{tb_name}({field_str}) VALUES ({placeholders})"

        self.logger.info(f"[batch_insert] SQL template: {sql}", stacklevel=3)

        data_size = len(data_list)
        if batch_size <= 0:
            batch_size = 3000

        for batch_start in range(0, data_size, batch_size):
            batch_end = min(batch_start + batch_size, data_size)
            batch = data_list[batch_start:batch_end]

            params_list = []
            for data_dict in batch:
                if not isinstance(data_dict, dict):
                    data_dict = data_dict.__dict__
                values = tuple(data_dict[key] for key in field_list)
                params_list.append(values)

            self._execute_many(sql, params_list)

    def foreach_insert(
        self,
        data_list: List[Any],
        db_name: str,
        tb_name: str,
        ignore_error: bool = False,
        auto_keys: List[str] = None
    ) -> None:
        """
        逐条插入数据（内部使用参数化查询，支持异常忽略）

        Args:
            data_list: 字典列表或对象列表
            db_name: 数据库名
            tb_name: 表名
            ignore_error: 是否忽略插入错误
            auto_keys: 自增字段列表
        """
        if auto_keys is None:
            auto_keys = ["id", "fid"]

        if not data_list:
            self.logger.info(f"[foreach_insert] No data to insert: {db_name}.{tb_name}", stacklevel=3)
            return

        self.logger.info(f"[foreach_insert] Data size: {len(data_list)}", stacklevel=3)

        if len(data_list) >= 50000:
            self.logger.warning(
                f"[foreach_insert] Too much data for this method: {len(data_list)}",
                stacklevel=3
            )

        # 获取字段列表
        first = data_list[0]
        if not isinstance(first, dict):
            first = first.__dict__

        field_list = [key for key in first.keys() if key.lower() not in auto_keys]
        field_str = ", ".join(field_list)
        placeholders = ", ".join(["%s"] * len(field_list))
        sql = f"INSERT INTO {db_name}.{tb_name}({field_str}) VALUES ({placeholders})"

        for data_dict in data_list:
            if not isinstance(data_dict, dict):
                data_dict = data_dict.__dict__

            params = tuple(data_dict[key] for key in field_list)

            try:
                self.update(sql, params=params)
            except Exception as error:
                self.logger.error(f"[foreach_insert] Failed to insert: {error}", stacklevel=3)
                if not ignore_error:
                    raise error

    def batch_replace(
        self,
        data_list: List[Any],
        db_name: str,
        tb_name: str,
        batch_size: int = 3000,
        auto_keys: List[str] = None
    ) -> None:
        """
        批量替换数据（REPLACE INTO，内部使用参数化查询）

        Args:
            data_list: 字典列表或对象列表
            db_name: 数据库名
            tb_name: 表名
            batch_size: 每批记录数
            auto_keys: 自增字段列表
        """
        if auto_keys is None:
            auto_keys = ["id", "fid"]

        if not data_list:
            self.logger.info(f"[batch_replace] No data to replace: {db_name}.{tb_name}", stacklevel=3)
            return

        self.logger.info(f"[batch_replace] Data size: {len(data_list)}", stacklevel=3)

        # 获取字段列表
        first = data_list[0]
        if not isinstance(first, dict):
            first = first.__dict__

        field_list = [key for key in first.keys() if key.lower() not in auto_keys]
        field_str = ", ".join(field_list)
        placeholders = ", ".join(["%s"] * len(field_list))
        sql = f"REPLACE INTO {db_name}.{tb_name}({field_str}) VALUES ({placeholders})"

        self.logger.info(f"[batch_replace] SQL template: {sql}", stacklevel=3)

        data_size = len(data_list)
        if batch_size <= 0:
            batch_size = 3000

        for batch_start in range(0, data_size, batch_size):
            batch_end = min(batch_start + batch_size, data_size)
            batch = data_list[batch_start:batch_end]

            params_list = []
            for data_dict in batch:
                if not isinstance(data_dict, dict):
                    data_dict = data_dict.__dict__
                values = tuple(data_dict[key] for key in field_list)
                params_list.append(values)

            self._execute_many(sql, params_list)

    def foreach_replace(
        self,
        data_list: List[Any],
        db_name: str,
        tb_name: str,
        ignore_error: bool = False,
        auto_keys: List[str] = None
    ) -> None:
        """
        逐条替换数据（内部使用参数化查询，支持异常忽略）

        Args:
            data_list: 字典列表或对象列表
            db_name: 数据库名
            tb_name: 表名
            ignore_error: 是否忽略错误
            auto_keys: 自增字段列表
        """
        if auto_keys is None:
            auto_keys = ["id", "fid"]

        if not data_list:
            self.logger.info(f"[foreach_replace] No data to replace: {db_name}.{tb_name}", stacklevel=3)
            return

        self.logger.info(f"[foreach_replace] Data size: {len(data_list)}", stacklevel=3)

        if len(data_list) >= 50000:
            self.logger.warning(
                f"[foreach_replace] Too much data for this method: {len(data_list)}",
                stacklevel=3
            )

        # 获取字段列表
        first = data_list[0]
        if not isinstance(first, dict):
            first = first.__dict__

        field_list = [key for key in first.keys() if key.lower() not in auto_keys]
        field_str = ", ".join(field_list)
        placeholders = ", ".join(["%s"] * len(field_list))
        sql = f"REPLACE INTO {db_name}.{tb_name}({field_str}) VALUES ({placeholders})"

        for data_dict in data_list:
            if not isinstance(data_dict, dict):
                data_dict = data_dict.__dict__

            params = tuple(data_dict[key] for key in field_list)

            try:
                self.update(sql, params=params)
            except Exception as error:
                self.logger.error(f"[foreach_replace] Failed to replace: {error}", stacklevel=3)
                if not ignore_error:
                    raise error

    def batch_delete(self, sql: str, params: Params = None, batch_size: int = 30000) -> DBResult:
        """
        分批删除数据

        Args:
            sql: DELETE语句（可包含 %s 占位符）
            params: SQL参数，与 %s 占位符对应
            batch_size: 每批删除的记录数

        Returns:
            DBResult实例

        Raises:
            DBError: 非DELETE语句时抛出

        Example:
            # 传统方式
            >>> db.batch_delete(f"DELETE FROM users WHERE age < {min_age}")

            # 参数化查询
            >>> db.batch_delete("DELETE FROM users WHERE age < %s", params=(min_age,))
        """
        sql = sql.replace("\n", " ").strip()

        # SQL 校验
        if not sql.lower().startswith("delete"):
            raise DBError(message=f"not delete sql! {sql}")

        if sql.endswith(";"):
            sql = sql[:-1]

        if "limit" not in sql.lower():
            sql += f" LIMIT {batch_size}"

        count = 0
        while count < 100000:
            count += 1
            log_msg = Xutil.simple_str(sql)[:8192]
            if params is not None:
                log_msg += f" | params: {str(params)[:512]}"
            self.logger.info(log_msg, stacklevel=3)
            db_ret = self._execute(sql, params=params, need_fetch=True, auto_commit=True)
            if db_ret.row_count <= 0:
                break

        return self.local_thread_data.db_ret

    def commit(self) -> DBResult:
        """提交事务"""
        try:
            self.local_thread_data.connection.commit()
        except mysql.connector.Error as error:
            raise DBError(message=str(error))
        return DBResult()

    def close(self) -> None:
        """关闭连接"""
        self.local_thread_data.cursor.close()
        self.local_thread_data.connection.close()

    def get_character_set(self) -> Dict[str, str]:
        """
        查询数据库字符编码设置

        Returns:
            字符编码配置字典
        """
        sql = "SHOW VARIABLES LIKE 'character_set%%'"
        ret = self.query(sql)

        if not ret.data:
            self.logger.info(f"DB result is empty: {ret.data}", stacklevel=3)
            return {}

        result = {}
        for item in ret.data:
            key = item["Variable_name"]
            value = item["Value"]
            result[key] = value

        return result

    def update_no_commit(self, sql: str, params: Params = None) -> DBResult:
        """
        执行更新SQL但不自动提交（用于事务）

        Args:
            sql: 更新SQL语句（可包含 %s 占位符）
            params: SQL参数，与 %s 占位符对应

        Returns:
            DBResult实例

        Example:
            # 事务操作
            >>> db.update_no_commit("UPDATE accounts SET balance = balance - %s WHERE id = %s", params=(100, 1))
            >>> db.update_no_commit("UPDATE accounts SET balance = balance + %s WHERE id = %s", params=(100, 2))
            >>> db.commit()
        """
        simple_sql = Xutil.simple_str(sql)
        if len(simple_sql) > 8192:
            simple_sql = f"{simple_sql[:8192]} ..."
        if params is not None:
            simple_sql += f" | params: {str(params)[:512]}"
        self.logger.info(simple_sql, stacklevel=3)

        # SQL 校验
        _ALLOWED_PREFIXES = ("delete", "insert", "replace", "update", "alter", "set")
        sql_lower = sql.strip().lower()
        if not sql_lower.startswith(_ALLOWED_PREFIXES):
            raise DBError(message=f"not delete|insert|replace|update|alter|set sql! {sql}")
        if sql_lower.startswith("delete") or sql_lower.startswith("set"):
            self.logger.info(Xutil.simple_str(sql), stacklevel=3)

        # 中文编码处理（仅对无参数的拼接SQL生效）
        if params is None and Xutil.contains_chinese(sql) and self.charset.lower() == "latin1":
            sql = sql.encode("gbk", "ignore")

        return self._execute(sql, params=params, need_fetch=False, auto_commit=False)


if __name__ == "__main__":
    pass



"""
日志管理工具模块

提供统一的日志管理功能，支持：
- 控制台日志输出
- 文件日志输出（按天轮转）
- 多级别日志控制
- 线程安全的单例模式
- 配置文件加载

Example:
    >>> logger = Xlog.get_logger("config.ini")
    >>> logger.info("Application started")
    >>> logger.error("An error occurred", exc_info=True)
"""
import os
import logging
import threading
from typing import Optional, Dict, Any, Union

from xsdk.config_reader import ConfigReader
from logging.handlers import TimedRotatingFileHandler

# 全局线程锁，用于单例模式
_lock = threading.Lock()

# 默认日志格式
DEFAULT_LOG_FORMAT = (
    "[%(asctime)s.%(msecs)03d %(levelname)s] "
    "[%(thread)d] (%(filename)s:%(lineno)d) %(message)s"
)
DEFAULT_DATE_FORMAT = "%Y-%m-%d %H:%M:%S"


class XlogConfig:
    """
    日志配置类

    封装日志相关的配置参数。

    Attributes:
        log_level: 全局日志级别
        console_log_flag: 是否输出到控制台
        file_log_flag: 是否输出到文件
        file_log_path: 日志文件目录
        file_log_name: 日志文件名

    Example:
        >>> config = XlogConfig(
        ...     log_level=logging.DEBUG,
        ...     console_log_flag=True,
        ...     file_log_flag=True,
        ...     file_log_path="./logs"
        ... )
    """

    def __init__(
        self,
        log_level: int = logging.DEBUG,
        console_log_flag: bool = True,
        file_log_flag: bool = False,
        file_log_path: str = "../log",
        file_log_name: str = "xlog.log"
    ):
        """
        初始化日志配置

        Args:
            log_level: 全局日志级别
            console_log_flag: 是否输出到控制台
            file_log_flag: 是否输出到文件
            file_log_path: 日志文件目录
            file_log_name: 日志文件名
        """
        self.log_level: int = log_level
        self.console_log_flag: bool = console_log_flag
        self.file_log_flag: bool = file_log_flag
        self.file_log_path: str = file_log_path
        self.file_log_name: str = file_log_name

    def __str__(self) -> str:
        """
        返回配置的字符串表示

        Returns:
            配置字典的字符串形式
        """
        return str(self.__dict__)

    @staticmethod
    def get_config(conf_file: str, conf_section: str = "xlog") -> "XlogConfig":
        """
        从配置文件加载日志配置

        Args:
            conf_file: 配置文件路径
            conf_section: 配置Section名称

        Returns:
            XlogConfig实例
        """
        config_reader = ConfigReader.get_instance(conf_file)
        log_level = config_reader.get(conf_section, "log_level")
        console_log_flag = config_reader.get(conf_section, "console_log_flag")
        file_log_flag = config_reader.get(conf_section, "file_log_flag")
        file_log_path = config_reader.get(conf_section, "file_log_path")
        file_log_name = config_reader.get(conf_section, "file_log_name")

        # 转换日志级别
        log_level = XlogConfig.get_log_level(log_level)

        # 转换布尔值
        console_log_flag = str(console_log_flag).lower() in ["y", "true"]
        file_log_flag = str(file_log_flag).lower() in ["y", "true"]

        # 至少开启一种日志模式
        if not console_log_flag and not file_log_flag:
            console_log_flag = True

        # 创建日志目录
        if file_log_flag and not os.path.exists(file_log_path):
            os.makedirs(file_log_path)

        return XlogConfig(
            log_level=log_level,
            console_log_flag=console_log_flag,
            file_log_flag=file_log_flag,
            file_log_path=file_log_path,
            file_log_name=file_log_name
        )

    @staticmethod
    def parse_dict(data_dict: Optional[Dict[str, Any]]) -> "XlogConfig":
        """
        从字典解析日志配置

        Args:
            data_dict: 配置字典

        Returns:
            XlogConfig实例
        """
        log_conf = XlogConfig()
        if data_dict is None or len(data_dict) == 0:
            return log_conf

        # 更新配置属性
        for key, value in data_dict.items():
            if key in log_conf.__dict__:
                log_conf.__dict__[key] = value

        # 转换布尔值
        log_conf.console_log_flag = str(log_conf.console_log_flag).lower() in ["y", "true"]
        log_conf.file_log_flag = str(log_conf.file_log_flag).lower() in ["y", "true"]

        # 至少开启一种日志模式
        if not log_conf.console_log_flag and not log_conf.file_log_flag:
            log_conf.console_log_flag = True

        # 创建日志目录
        if log_conf.file_log_flag and not os.path.exists(log_conf.file_log_path):
            os.makedirs(log_conf.file_log_path)

        return log_conf

    @staticmethod
    def get_log_level(log_level: Union[str, int]) -> int:
        """
        获取日志级别的数值

        Args:
            log_level: 日志级别（字符串或整数）

        Returns:
            日志级别数值
        """
        if isinstance(log_level, int):
            return log_level

        level = logging.getLevelName(str(log_level).upper())
        if isinstance(level, int):
            return level
        return logging.INFO


class Xlog:
    """
    日志管理类

    提供线程安全的日志记录功能，支持控制台和文件输出。

    Attributes:
        logger: 类级别的单例logger实例
        log_conf: 日志配置

    Example:
        >>> logger = Xlog.get_logger("config.ini")
        >>> logger.info("Processing data...")
        >>> logger.error("Error occurred", exc_info=True)
    """

    logger: Optional["Xlog"] = None

    def __init__(self, log_conf: Union[XlogConfig, Dict[str, Any]]):
        """
        初始化日志管理器

        Args:
            log_conf: 日志配置对象或字典
        """
        if isinstance(log_conf, dict):
            log_conf = XlogConfig.parse_dict(log_conf)
        self.log_conf: XlogConfig = log_conf
        self._logger: Optional[logging.Logger] = None
        self._init()

    def _init(self) -> None:
        """
        初始化日志处理器

        配置控制台和文件日志处理器。
        """
        self._logger = logging.getLogger("Xlog")
        self._logger.handlers.clear()  # 添加这行
        self._logger.setLevel(self.log_conf.log_level)

        formatter = logging.Formatter(
            fmt=DEFAULT_LOG_FORMAT,
            datefmt=DEFAULT_DATE_FORMAT
        )

        # 控制台日志处理器
        if self.log_conf.console_log_flag:
            handler_console = logging.StreamHandler()
            handler_console.setLevel(self.log_conf.log_level)
            handler_console.setFormatter(formatter)
            self._logger.addHandler(handler_console)

        # 文件日志处理器（按天轮转，保留45天）
        if self.log_conf.file_log_flag:
            log_file = f"{self.log_conf.file_log_path}/{self.log_conf.file_log_name}"
            handler_file = TimedRotatingFileHandler(
                log_file,
                when="D",
                interval=1,
                backupCount=45
            )
            handler_file.setLevel(self.log_conf.log_level)
            handler_file.setFormatter(formatter)
            self._logger.addHandler(handler_file)

    @staticmethod
    def get_logger(conf_file: str, conf_section: str = "xlog") -> "Xlog":
        """
        获取日志管理器单例

        线程安全的单例模式，确保全局只有一个日志实例。

        Args:
            conf_file: 配置文件路径
            conf_section: 配置Section名称

        Returns:
            Xlog单例实例
        """
        if Xlog.logger is None:
            with _lock:
                if Xlog.logger is None:
                    log_conf = XlogConfig.get_config(conf_file, conf_section)
                    Xlog.logger = Xlog(log_conf)
        return Xlog.logger

    @staticmethod
    def get_default_logger(log_level: str = "INFO") -> logging.Logger:
        """
        获取默认的Python logger

        Args:
            log_level: 日志级别

        Returns:
            标准logging.Logger实例
        """
        return logging.getLogger("DefaultLogger")

    def set_level(self, log_level: str = "INFO") -> None:
        """
        动态设置日志级别

        Args:
            log_level: 日志级别字符串
        """
        level = XlogConfig.get_log_level(log_level)
        self._logger.setLevel(level)

    @staticmethod
    def _get_stack_level(kwargs: Dict[str, Any]) -> int:
        """
        从kwargs中提取stacklevel参数

        Args:
            kwargs: 关键字参数字典

        Returns:
            stacklevel值
        """
        sl1 = kwargs.pop("stacklevel", None)
        sl2 = kwargs.pop("stack_level", None)
        stack_level = sl1 if sl1 is not None else sl2

        try:
            stack_level = int(str(stack_level))
        except (ValueError, TypeError):
            stack_level = 2

        return max(1, stack_level)

    def debug(self, msg: str, *args: Any, **kwargs: Any) -> None:
        """
        记录DEBUG级别日志

        Args:
            msg: 日志消息
            *args: 格式化参数
            **kwargs: 关键字参数
        """
        stack_level = Xlog._get_stack_level(kwargs)
        self._logger.debug(msg, *args, stacklevel=stack_level, **kwargs)

    def info(self, msg: str, *args: Any, **kwargs: Any) -> None:
        """
        记录INFO级别日志

        Args:
            msg: 日志消息
            *args: 格式化参数
            **kwargs: 关键字参数
        """
        stack_level = Xlog._get_stack_level(kwargs)
        self._logger.info(msg, *args, stacklevel=stack_level, **kwargs)

    def warn(self, msg: str, *args: Any, **kwargs: Any) -> None:
        """
        记录WARNING级别日志（warn别名）

        Args:
            msg: 日志消息
            *args: 格式化参数
            **kwargs: 关键字参数
        """
        stack_level = Xlog._get_stack_level(kwargs)
        self._logger.warning(msg, *args, stacklevel=stack_level, **kwargs)

    def warning(self, msg: str, *args: Any, **kwargs: Any) -> None:
        """
        记录WARNING级别日志

        Args:
            msg: 日志消息
            *args: 格式化参数
            **kwargs: 关键字参数
        """
        stack_level = Xlog._get_stack_level(kwargs)
        self._logger.warning(msg, *args, stacklevel=stack_level, **kwargs)

    def error(self, msg: str, *args: Any, **kwargs: Any) -> None:
        """
        记录ERROR级别日志

        Args:
            msg: 日志消息
            *args: 格式化参数
            **kwargs: 关键字参数
        """
        stack_level = Xlog._get_stack_level(kwargs)
        self._logger.error(msg, *args, stacklevel=stack_level, **kwargs)

    def fatal(self, msg: str, *args: Any, **kwargs: Any) -> None:
        """
        记录FATAL级别日志

        Args:
            msg: 日志消息
            *args: 格式化参数
            **kwargs: 关键字参数
        """
        stack_level = Xlog._get_stack_level(kwargs)
        self._logger.fatal(msg, *args, stacklevel=stack_level, **kwargs)

    def exception(
        self,
        msg: str,
        *args: Any,
        exc_info: bool = True,
        **kwargs: Any
    ) -> None:
        """
        记录异常日志

        Args:
            msg: 日志消息
            *args: 格式化参数
            exc_info: 是否包含异常信息
            **kwargs: 关键字参数
        """
        stack_level = Xlog._get_stack_level(kwargs)
        self._logger.exception(msg, *args, exc_info=exc_info, stacklevel=stack_level, **kwargs)


if __name__ == "__main__":
    pass



"""
图表工具模块

提供图表生成和图片处理功能，支持：
- 图片与Base64互转
- 柱状图生成
- 折线图生成（支持双Y轴）
- 同比折线图

Example:
    >>> ChartUtil.get_bar_chart("output.png", {"A": 1, "B": 2})
    >>> ChartUtil.get_line_chart("line.png", ["Q1", "Q2"], {"data": [10, 20]})
"""
import base64
import logging
from typing import Dict, List, Optional

from matplotlib import pyplot as plt

logger = logging.getLogger(__name__)


class ChartUtil:
    """
    图表工具类

    提供静态方法进行图表生成和图片Base64编码转换。

    Example:
        >>> data = {"A": 100, "B": 200, "C": 150}
        >>> ChartUtil.get_bar_chart("chart.png", data)
    """

    # 默认颜色列表
    COLORS: List[str] = [
        "Blue", "Red", "Black", "Yellow",
        "Green", "Cyan", "Magenta", "Orange"
    ]

    @staticmethod
    def img_to_base64(img_file_path: str) -> bytes:
        """
        将图片文件转换为Base64编码

        Args:
            img_file_path: 图片文件路径

        Returns:
            Base64编码的字节数据
        """
        with open(img_file_path, "rb") as file_reader:
            return base64.b64encode(file_reader.read())

    @staticmethod
    def base64_to_img(str_base64: bytes, img_file_path: str) -> None:
        """
        将Base64编码转换为图片文件

        Args:
            str_base64: Base64编码的字节数据
            img_file_path: 输出图片文件路径
        """
        img = base64.b64decode(str_base64)
        with open(img_file_path, "wb") as file_writer:
            file_writer.write(img)

    @staticmethod
    def get_bar_chart(
        img_path: str,
        data_dict: Dict[str, float],
        width: int = 20,
        height: int = 5
    ) -> None:
        """
        生成柱状图

        Args:
            img_path: 图片保存路径
            data_dict: 数据字典，键为标签，值为数值
            width: 图表宽度
            height: 图表高度

        Example:
            >>> data = {"A": 100, "B": 200, "C": 150}
            >>> ChartUtil.get_bar_chart("bar_chart.png", data)
        """
        keys = sorted(list(data_dict.keys()))
        data_list = [data_dict[key] for key in keys]

        plt.figure(figsize=(width, height))
        plt.bar(range(len(data_list)), data_list)
        plt.xticks(range(len(keys)), keys)
        plt.savefig(img_path)
        plt.close()

    @staticmethod
    def get_line_chart(
        img_path: str,
        x_list: List[str],
        master_dict: Dict[str, List[float]],
        slave_dict: Optional[Dict[str, List[float]]] = None,
        x_label: str = "",
        y_label: str = "",
        title: str = "",
        width: int = 12,
        height: int = 3,
        bottom: float = 0.25
    ) -> None:
        """
        生成折线图

        支持双Y轴，主轴数据在左侧，副轴数据在右侧。

        Args:
            img_path: 图片保存路径
            x_list: X轴坐标标签列表
            master_dict: 主Y轴数据字典，格式为 {"系列名": [数值列表]}
            slave_dict: 副Y轴数据字典（可选）
            x_label: X轴标签
            y_label: Y轴标签
            title: 图表标题
            width: 图表宽度
            height: 图表高度
            bottom: 底部边距比例

        Example:
            >>> x = ["Q1", "Q2", "Q3", "Q4"]
            >>> master = {"Sales": [100, 150, 120, 180]}
            >>> slave = {"Growth": [0.1, 0.5, -0.2, 0.5]}
            >>> ChartUtil.get_line_chart("line.png", x, master, slave)
        """
        if slave_dict is None:
            slave_dict = {}

        plt.figure(figsize=(width, height))
        plt.subplots_adjust(bottom=bottom)

        # X轴坐标
        xx = range(len(x_list))
        plt.xlabel(x_label)
        plt.xticks(rotation=90, fontsize=8)
        plt.xticks(list(xx), x_list)

        # Y轴网格
        plt.grid(axis="y")

        # 主Y轴数据
        idx_color = 0
        for key in master_dict:
            color_idx = idx_color % len(ChartUtil.COLORS)
            yy = master_dict[key]
            plt.plot(xx, yy, label=key, color=ChartUtil.COLORS[color_idx])
            idx_color += 1

        # 副Y轴数据
        for key in slave_dict:
            color_idx = idx_color % len(ChartUtil.COLORS)
            yy = slave_dict[key]
            plt.twinx()
            plt.plot(xx, yy, label=key, color=ChartUtil.COLORS[color_idx])
            idx_color += 1

        plt.ylabel(y_label)
        plt.title(title)
        plt.savefig(img_path, bbox_inches="tight", pad_inches=0.1)
        plt.close()

    @staticmethod
    def get_line_chart_compare_base(
        date_list: List[str],
        data_dict: Dict[str, List[float]],
        save_file_path: str
    ) -> None:
        """
        生成同比折线图

        以每个系列的首个值为基数，计算相对比例。

        Args:
            date_list: 日期/X轴标签列表
            data_dict: 数据字典，格式为 {"系列名": [数值列表]}
            save_file_path: 图片保存路径

        Example:
            >>> dates = ["2024-01", "2024-02", "2024-03"]
            >>> data = {"A": [100, 120, 150], "B": [200, 180, 220]}
            >>> ChartUtil.get_line_chart_compare_base(dates, data, "compare.png")
        """
        normalized_dict: Dict[str, List[float]] = {}

        for key in data_dict:
            data_list = data_dict[key]
            base = data_list[0] if data_list else 0

            tmp_list: List[float] = []
            for value in data_list:
                if base == 0 and value == 0:
                    tmp_list.append(0.0)
                elif base == 0 and value >= 0:
                    tmp_list.append(1.0)
                    base = value
                else:
                    tmp_list.append(value / base)
            normalized_dict[key] = tmp_list

        ChartUtil.get_line_chart(save_file_path, date_list, normalized_dict)


if __name__ == "__main__":
    # 测试柱状图
    test_dict = {"A": 1, "B": 2, "C": 3}
    test_path = "../../data/test.png"
    ChartUtil.get_bar_chart(test_path, test_dict)



"""
腾讯云COS存储客户端模块

提供腾讯云对象存储服务的封装，支持：
- 文件上传/下载/删除
- 文件列表查询
- 配置文件方式初始化
"""
import os
import logging
from typing import Optional, List, Dict, Any

from qcloud_cos import CosConfig
from qcloud_cos import CosS3Client

from xsdk.file_util import FileUtil
from xsdk.config_reader import ConfigReader

logger = logging.getLogger(__name__)


class LocalCosConfig:
    """
    COS本地配置类

    存储腾讯云COS服务的连接配置信息。

    Attributes:
        secret_id: 腾讯云SecretId
        secret_key: 腾讯云SecretKey
        region: 存储桶所在地域
        bucket: 存储桶名称
        scheme: 协议类型（http/https）
        token: 临时密钥token（可选）
    """

    def __init__(
        self,
        secret_id: str,
        secret_key: str,
        region: str,
        bucket: str,
        scheme: str = "https",
        token: Optional[str] = None
    ):
        self.secret_id = secret_id
        self.secret_key = secret_key
        self.region = region
        self.bucket = bucket
        self.scheme = scheme
        self.token = token

    @staticmethod
    def get_config(conf_file: str, conf_section: str = "cos") -> "LocalCosConfig":
        """
        从配置文件读取COS配置

        Args:
            conf_file: 配置文件路径
            conf_section: 配置节名称

        Returns:
            LocalCosConfig实例
        """
        config_reader = ConfigReader.get_instance(conf_file)
        secret_id = config_reader.get(conf_section, "secret_id")
        secret_key = config_reader.get(conf_section, "secret_key")
        region = config_reader.get(conf_section, "region")
        bucket = config_reader.get(conf_section, "bucket")
        scheme = config_reader.get(conf_section, "scheme") or "https"
        token = config_reader.get(conf_section, "token")

        return LocalCosConfig(secret_id, secret_key, region, bucket, scheme, token)


class CosClient:
    """
    腾讯云COS客户端封装类

    提供文件的上传、下载、删除和列表查询功能。

    Attributes:
        bucket: 存储桶名称
        config: COS配置对象
        client: COS客户端实例
        logger: 日志记录器

    Example:
        >>> client = CosClient.get_client("config.ini")
        >>> client.upload_file("remote/path/file.txt", "/local/path/file.txt")
        >>> client.download_file("remote/path/file.txt", "/local/path/file.txt")
    """

    def __init__(
        self,
        bucket: str,
        config: CosConfig,
        client: CosS3Client,
        logger: logging.Logger = logger
    ):
        self.bucket = bucket
        self.config = config
        self.client = client
        self.logger = logger

    @staticmethod
    def get_client(
        conf_file: str,
        conf_section: str = "cos",
        logger: logging.Logger = logger
    ) -> "CosClient":
        """
        从配置文件创建COS客户端

        Args:
            conf_file: 配置文件路径
            conf_section: 配置节名称
            logger: 日志记录器

        Returns:
            CosClient实例
        """
        local_config = LocalCosConfig.get_config(conf_file, conf_section)

        bucket = local_config.bucket
        cos_config = CosConfig(
            Region=local_config.region,
            SecretId=local_config.secret_id,
            SecretKey=local_config.secret_key,
            Token=local_config.token,
            Scheme=local_config.scheme
        )
        client = CosS3Client(cos_config)

        return CosClient(bucket, cos_config, client, logger)

    def upload_file(self, file_cos: str, file_local: str) -> bool:
        """
        上传本地文件到COS

        Args:
            file_cos: COS上的目标路径
            file_local: 本地文件路径

        Returns:
            上传是否成功
        """
        if not os.path.exists(file_local) or not os.path.isfile(file_local):
            self.logger.warning(f"upload_file: invalid local file: {file_local}")
            return False

        try:
            with open(file_local, 'rb') as fp:
                response = self.client.put_object(
                    Bucket=self.bucket,
                    Body=fp,
                    Key=file_cos,
                    StorageClass='STANDARD',
                    EnableMD5=False
                )
                self.logger.info(f"upload_file: {file_cos} | {response}")
            return True
        except Exception as e:
            self.logger.error(f"upload_file failed: {file_cos} | {e}")
            return False

    def download_file(self, file_cos: str, file_local: str, overwrite: bool = False) -> bool:
        """
        从COS下载文件到本地

        Args:
            file_cos: COS上的文件路径
            file_local: 本地目标路径
            overwrite: 是否覆盖已存在的文件

        Returns:
            下载是否成功
        """
        if os.path.exists(file_local):
            if overwrite:
                FileUtil.rm_file(file_local)
            else:
                self.logger.info(f"download_file: file already exists: {file_local}")
                return False

        try:
            response = self.client.get_object(
                Bucket=self.bucket,
                Key=file_cos
            )
            response['Body'].get_stream_to_file(file_local)
            self.logger.info(f"download_file: {file_cos} -> {file_local}")
            return True
        except Exception as e:
            self.logger.error(f"download_file failed: {file_cos} | {e}")
            return False

    def delete_file(self, file_cos: str) -> bool:
        """
        删除COS上的文件

        Args:
            file_cos: COS上的文件路径

        Returns:
            删除是否成功
        """
        try:
            response = self.client.delete_object(
                Bucket=self.bucket,
                Key=file_cos
            )
            self.logger.info(f"delete_file: {file_cos} | {response}")
            return True
        except Exception as e:
            self.logger.error(f"delete_file failed: {file_cos} | {e}")
            return False

    def list_files(self, prefix: str = "") -> List[Dict[str, Any]]:
        """
        列出COS存储桶中的文件

        Args:
            prefix: 文件路径前缀（用于过滤）

        Returns:
            文件信息字典列表
        """
        try:
            response = self.client.list_objects(
                Bucket=self.bucket,
                Prefix=prefix
            )
            if "Contents" in response and isinstance(response["Contents"], list):
                return response["Contents"]
            else:
                self.logger.info(f"list_files: no files found with prefix '{prefix}'")
                return []
        except Exception as e:
            self.logger.error(f"list_files failed: {e}")
            return []


if __name__ == '__main__':
    pass



"""
文件操作工具模块

提供常用的文件操作功能，支持：
- 文件读写、删除、复制
- 文件MD5计算
- 目录操作
- 压缩文件处理（tar、zip）
- 文件列表查询
- 本地MD5缓存管理
"""
import os
import time
import shutil
import hashlib
from hashlib import md5
from typing import List, Optional


class FileUtil:
    """
    文件操作工具类

    提供静态方法进行文件和目录操作，包含安全检查机制防止误删系统文件。

    Example:
        >>> FileUtil.write_file("/path/to/file.txt", ["line1", "line2"])
        >>> content = FileUtil.get_file_content("/path/to/file.txt")
        >>> file_list = FileUtil.get_file_list("/path/to/dir", child=True)
    """

    @staticmethod
    def write_file(file_path: str, data_list: List[str]) -> None:
        """
        将列表数据按行写入文件

        Args:
            file_path: 文件路径
            data_list: 数据列表，每个元素为一行
        """
        with open(file_path, 'w+', encoding='utf-8') as f:
            for line in data_list:
                f.write(str(line) + "\n")

    @staticmethod
    def split_file(file_path: str, target_path: str, line_count: int = 10000000) -> None:
        """
        按行数分割大文件

        Args:
            file_path: 源文件路径
            target_path: 目标目录路径
            line_count: 每个分割文件的行数
        """
        if not FileUtil.file_exists(file_path):
            return

        file_name = FileUtil.get_file_name(file_path)
        file_name = f"{file_name[:file_name.find('.')]}_split_"

        cmd_split = f"cd {target_path}; split -l {line_count} {file_path} {file_name};"
        os.system(cmd_split)
        FileUtil.rm_file(file_path)

    @staticmethod
    def untar_file(file_path: str, target_path: str) -> None:
        """
        解压tar.gz文件

        Args:
            file_path: tar.gz文件路径
            target_path: 解压目标目录
        """
        if os.path.exists(target_path):
            FileUtil.rm_file(target_path)
        FileUtil.mk_dirs(target_path)
        cmd_untar = f"tar -zxvf {file_path} -C {target_path}"
        os.system(cmd_untar)

    @staticmethod
    def unzip_file(file_path: str, target_path: str) -> None:
        """
        解压zip文件

        Args:
            file_path: zip文件路径
            target_path: 解压目标目录
        """
        if not os.path.exists(target_path):
            FileUtil.mk_dirs(target_path)

        if str(file_path).endswith(".zip"):
            cmd_unzip = f"unzip -o {file_path} -d {target_path}"
            os.system(cmd_unzip)
        else:
            print(f"Not a zip file: {file_path}")

    @staticmethod
    def file_exists(file_path: str) -> bool:
        """
        检查文件是否存在且有效

        Args:
            file_path: 文件路径

        Returns:
            文件存在且非空返回True
        """
        if not os.path.exists(file_path):
            print(f"File not exist: {file_path}")
            return False
        if not os.path.isfile(file_path):
            print(f"Not a file: {file_path}")
            return False
        if os.path.getsize(file_path) <= 0:
            print(f"File is empty: {file_path}")
            return False
        return True

    @staticmethod
    def build_file(path: str, file_name: str) -> str:
        """
        构建文件完整路径

        Args:
            path: 目录路径
            file_name: 文件名

        Returns:
            规范化的文件路径
        """
        result = f"{path}/{file_name}"
        while "//" in result:
            result = result.replace("//", "/")
        return result

    @staticmethod
    def get_file_md5(file_path: str) -> str:
        """
        计算文件的MD5值

        Args:
            file_path: 文件路径

        Returns:
            文件的MD5哈希值
        """
        file_path = FileUtil.clean_path(file_path)
        with open(file_path, 'rb') as f:
            file_md5 = hashlib.md5(f.read()).hexdigest()
        return file_md5

    @staticmethod
    def get_file_name(file_path: str, remove_suffix: bool = False) -> str:
        """
        从路径中提取文件名

        Args:
            file_path: 文件路径
            remove_suffix: 是否移除文件后缀

        Returns:
            文件名
        """
        if file_path is None:
            return ""

        file_path = FileUtil.clean_path(file_path)

        idx = file_path.rfind("/")
        if idx >= 0:
            file_name = file_path[idx + 1:]
        else:
            file_name = file_path

        if remove_suffix:
            dot_idx = file_name.rfind(".")
            if dot_idx > 0:
                file_name = file_name[:dot_idx]

        return file_name

    @staticmethod
    def get_file_list(
        file_path: str,
        child: bool = False,
        file_name_prefix: str = "",
        file_name_suffix: str = ""
    ) -> List[str]:
        """
        获取目录下的文件列表

        Args:
            file_path: 目录或文件路径
            child: 是否递归子目录
            file_name_prefix: 文件名前缀过滤
            file_name_suffix: 文件名后缀过滤

        Returns:
            文件路径列表
        """
        file_list: List[str] = []

        if not os.path.exists(file_path):
            print(f"Path not exist: {file_path}")
            return file_list

        file_path = FileUtil.clean_path(file_path)
        if os.path.isfile(file_path):
            return [file_path]

        for file_name in os.listdir(file_path):
            if file_name.startswith(".") or file_name.startswith("~"):
                continue

            temp_path = FileUtil.build_file(file_path, file_name)

            if os.path.isdir(temp_path):
                if child:
                    file_list.extend(
                        FileUtil.get_file_list(temp_path, child, file_name_prefix, file_name_suffix)
                    )
            elif os.path.isfile(temp_path):
                prefix_match = not file_name_prefix or file_name.startswith(file_name_prefix)
                suffix_match = not file_name_suffix or file_name.endswith(file_name_suffix)
                if prefix_match and suffix_match:
                    file_list.append(str(temp_path))

        return file_list

    @staticmethod
    def get_field_list(size: int) -> List[str]:
        """
        生成字段名列表

        Args:
            size: 字段数量

        Returns:
            格式为col_01, col_02...的字段名列表
        """
        if size <= 0:
            return []

        field_list = []
        for i in range(size):
            col_name = f"col_{str(i + 1).zfill(2)}"
            field_list.append(col_name)

        return field_list

    @staticmethod
    def get_date_current(date_format: str = "%Y-%m-%d %H:%M:%S") -> str:
        """
        获取当前日期时间字符串

        Args:
            date_format: 日期格式

        Returns:
            格式化的日期时间字符串
        """
        return time.strftime(date_format, time.localtime(time.time()))

    @staticmethod
    def is_empty(input_str: Optional[str]) -> bool:
        """
        检查字符串是否为空

        Args:
            input_str: 输入字符串

        Returns:
            字符串为None或空白时返回True
        """
        if input_str is None or str(input_str).strip() == "":
            return True
        return False

    @staticmethod
    def is_not_empty(input_str: Optional[str]) -> bool:
        """
        检查字符串是否非空

        Args:
            input_str: 输入字符串

        Returns:
            字符串非空时返回True
        """
        return not FileUtil.is_empty(input_str)

    @staticmethod
    def rm_file(file_path: str) -> bool:
        """
        安全删除文件或目录

        包含安全检查，防止误删系统文件。

        Args:
            file_path: 文件或目录路径

        Returns:
            删除是否成功

        Raises:
            UserWarning: 路径层级过浅时抛出
        """
        file_path = os.path.abspath(file_path)

        # 安全检查：限制目录层级，防止误删系统文件
        if file_path.count("/") <= 3 or len(file_path.strip()) <= 15:
            msg = f"No permission to remove {file_path}"
            print(msg)
            raise UserWarning(msg)

        if os.path.exists(file_path):
            if os.path.isdir(file_path):
                shutil.rmtree(file_path)
            elif os.path.isfile(file_path):
                os.remove(file_path)
            else:
                print(f"Illegal file: {file_path}")
                return False

        return True

    @staticmethod
    def mk_dirs(dir_path: str, is_file: bool = False) -> None:
        """
        创建目录

        Args:
            dir_path: 目录路径
            is_file: 如果为True，则dir_path被视为文件路径，创建其父目录
        """
        if is_file:
            dir_path = os.path.dirname(dir_path)
        if not os.path.exists(dir_path):
            os.makedirs(dir_path)

    @staticmethod
    def rebuild_file_path(file_path: str) -> str:
        """
        规范化文件路径，移除多余的斜杠

        Args:
            file_path: 文件路径

        Returns:
            规范化后的路径
        """
        if file_path is None:
            return ""

        temp_path = str(file_path).strip()
        if FileUtil.is_empty(temp_path):
            return ""

        while "//" in temp_path:
            temp_path = temp_path.replace("//", "/")

        return temp_path

    @staticmethod
    def get_file_content(file_path: str) -> str:
        """
        读取文件全部内容

        Args:
            file_path: 文件路径

        Returns:
            文件内容字符串
        """
        if file_path is None:
            return ""
        if not os.path.exists(file_path) or not os.path.isfile(file_path):
            return ""

        with open(file_path, 'r', encoding='utf-8') as f:
            return f.read()

    @staticmethod
    def get_dir_md5(file_path: str) -> str:
        """
        计算目录的MD5值（基于所有文件的MD5）

        Args:
            file_path: 目录路径

        Returns:
            目录的MD5哈希值
        """
        file_path = FileUtil.clean_path(file_path)
        file_list = FileUtil.get_file_list(file_path, child=True)

        md5_list = []
        for file_item in file_list:
            if file_item.startswith(".") or file_item.startswith("~"):
                continue
            file_md5 = FileUtil.get_file_md5(file_item)
            md5_list.append(file_md5)

        md5_list = sorted(md5_list)
        md5_str = "#".join(md5_list)
        return md5(md5_str.encode("utf-8")).hexdigest()

    @staticmethod
    def update_local_md5(file_path: str, local_file_info: str = "./local_file_info.csv") -> None:
        """
        更新本地MD5缓存

        Args:
            file_path: 文件或目录路径
            local_file_info: 缓存文件路径
        """
        FileUtil.remove_local_md5(file_path, local_file_info)
        FileUtil.add_local_md5(file_path, local_file_info)

    @staticmethod
    def add_local_md5(file_path: str, local_file_info: str = "./local_file_info.csv") -> None:
        """
        添加文件MD5到本地缓存

        Args:
            file_path: 文件或目录路径
            local_file_info: 缓存文件路径
        """
        if not os.path.exists(local_file_info):
            cmd_touch = f"touch {local_file_info}"
            os.popen(cmd_touch).readlines()

        file_path = FileUtil.clean_path(file_path)
        file_key = f"KEY_{md5(file_path.encode('utf-8')).hexdigest()}"
        file_md5 = FileUtil.get_dir_md5(file_path)
        data_time = time.time()

        cmd_add = f"echo {file_key}###{file_md5}###{data_time}###{file_path} >> {local_file_info}"
        os.popen(cmd_add).readlines()

    @staticmethod
    def remove_local_md5(file_path: str, local_file_info: str = "./local_file_info.csv") -> None:
        """
        从本地缓存中移除文件MD5

        Args:
            file_path: 文件或目录路径
            local_file_info: 缓存文件路径
        """
        file_path = FileUtil.clean_path(file_path)
        file_key = f"KEY_{md5(file_path.encode('utf-8')).hexdigest()}"

        cmd_remove = f"sed -i -e '/{file_key}/d' {local_file_info}"
        os.popen(cmd_remove).readlines()

    @staticmethod
    def get_local_md5(
        file_path: str,
        local_file_info: str = "./local_file_info.csv",
        timeout: int = 24 * 60 * 60
    ) -> Optional[str]:
        """
        从本地缓存获取文件MD5

        Args:
            file_path: 文件或目录路径
            local_file_info: 缓存文件路径
            timeout: 缓存超时时间（秒）

        Returns:
            MD5值，缓存不存在或过期时返回None
        """
        if not os.path.exists(local_file_info):
            return None

        file_path = FileUtil.clean_path(file_path)
        file_key = f"KEY_{md5(file_path.encode('utf-8')).hexdigest()}"

        cmd_grep = f"grep '{file_key}' {local_file_info}"
        ret_grep = os.popen(cmd_grep).readlines()

        if ret_grep is None or len(ret_grep) != 1:
            return None

        parts = str(ret_grep[0]).replace("\n", "").split("###")
        if len(parts) != 4:
            return None

        _, file_md5, file_time, _ = parts
        if time.time() - float(file_time) > timeout:
            FileUtil.remove_local_md5(file_path, local_file_info)
            return None

        return file_md5

    @staticmethod
    def check_local_md5(
        file_path: str,
        local_file_info: str = "./local_file_info.csv",
        timeout: int = 24 * 60 * 60
    ) -> bool:
        """
        检查文件MD5是否与本地缓存匹配

        Args:
            file_path: 文件或目录路径
            local_file_info: 缓存文件路径
            timeout: 缓存超时时间（秒）

        Returns:
            MD5匹配返回True
        """
        if not os.path.exists(local_file_info):
            return False

        file_path = FileUtil.clean_path(file_path)
        file_md5 = FileUtil.get_dir_md5(file_path)
        local_md5 = FileUtil.get_local_md5(file_path, local_file_info, timeout)

        return local_md5 is not None and local_md5 == file_md5

    @staticmethod
    def clean_path(file_path: str) -> str:
        """
        清理和规范化文件路径

        Args:
            file_path: 文件路径

        Returns:
            规范化后的绝对路径
        """
        file_path = str(os.path.abspath(file_path)).strip()
        while "//" in file_path:
            file_path = file_path.replace("//", "/")
        if file_path.endswith("/"):
            file_path = file_path[:-1]
        return file_path


if __name__ == "__main__":
    test_file = "./http_client.py"
    print(f"File exists: {os.path.isfile(test_file)}")

    test_file_not_exist = "./http_client_1.py"
    print(f"File exists: {os.path.isfile(test_file_not_exist)}")



"""
HTTP代理工具模块

提供带代理的HTTP请求功能，支持：
- 代理配置解析
- 多代理随机轮换
- GET/POST请求代理封装
- 无代理时自动降级

配置环境变量：
    export XSDK_PROXY="http://{usr}:{pwd}@{host}:{port}"
    多个代理用 ^^ 分隔：
    export XSDK_PROXY="http://usr1:pwd1@host1:port1^^http://usr2:pwd2@host2:port2"

代理配置来源（IP代理商）：
    https://proxymesh.com/account/dashboard/
"""
import time
import random
import logging
from typing import Optional, Dict, Any, List, Union

import requests

from xsdk.xutil import Xutil
from urllib.parse import urlparse, quote_plus

logger = logging.getLogger(__name__)


class ProxyConfig:
    """
    代理配置类

    解析和管理HTTP代理配置信息。

    Attributes:
        usr: 代理用户名
        pwd: 代理密码
        host: 代理主机
        port: 代理端口
        schema: 协议类型（http/https）

    Example:
        >>> config = ProxyConfig.parse_config("http://user:pass@proxy.example.com:8080")
        >>> proxies = {"http": config.to_str(), "https": config.to_str()}
    """

    def __init__(
        self,
        usr: str = "",
        pwd: str = "",
        host: str = "",
        port: int = 80,
        schema: str = "http"
    ):
        self.usr = usr
        self.pwd = pwd
        self.host = host
        self.port = port
        self.schema = schema

    def __str__(self) -> str:
        return str(self.__dict__)

    def to_str(self) -> str:
        """
        转换为代理URL字符串

        Returns:
            格式化的代理URL
        """
        return f"{self.schema}://{self.usr}:{self.pwd}@{self.host}:{self.port}"

    @staticmethod
    def parse_config(url: str) -> "ProxyConfig":
        """
        解析代理URL为配置对象

        Args:
            url: 代理URL字符串

        Returns:
            ProxyConfig实例
        """
        url_parsed = urlparse(url)
        proxy_config = ProxyConfig()
        proxy_config.usr = quote_plus(url_parsed.username or "")
        proxy_config.pwd = quote_plus(url_parsed.password or "")
        proxy_config.host = url_parsed.hostname or ""
        proxy_config.port = url_parsed.port or 80
        proxy_config.schema = url_parsed.scheme
        return proxy_config

    @staticmethod
    def get_config_list() -> List["ProxyConfig"]:
        """
        从环境变量获取代理配置列表

        Returns:
            ProxyConfig列表，配置不存在时返回空列表
        """
        proxy_str = Xutil.get_env_param("XSDK_PROXY")
        if Xutil.is_empty(proxy_str):
            logger.debug("XSDK_PROXY environment variable not set")
            return []

        proxy_list = proxy_str.strip().split("^^")
        if not proxy_list:
            logger.debug("XSDK_PROXY is empty")
            return []

        res_list = []
        for item in proxy_list:
            if item.strip():
                config = ProxyConfig.parse_config(item.strip())
                res_list.append(config)

        return res_list

    @staticmethod
    def get_config() -> Optional["ProxyConfig"]:
        """
        随机获取一个代理配置

        Returns:
            随机选择的ProxyConfig，无配置时返回None
        """
        proxy_list = ProxyConfig.get_config_list()
        if not proxy_list:
            return None

        idx = random.randint(0, len(proxy_list) - 1)
        return proxy_list[idx]


class HttpProxy:
    """
    HTTP代理请求类

    提供带代理支持的GET和POST请求方法，无代理时自动降级为直连。

    Example:
        >>> response = HttpProxy.do_get("https://api.example.com/data")
        >>> response = HttpProxy.do_post("https://api.example.com/data", json={"key": "value"})
    """

    @staticmethod
    def do_get(
        url: str,
        params: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
        cookies: Optional[Dict[str, str]] = None,
        logger: logging.Logger = logger,
        timeout: int = 150
    ) -> Optional[Union[str, int]]:
        """
        发送带代理的GET请求

        Args:
            url: 请求URL
            params: 查询参数
            headers: 请求头
            cookies: Cookie
            logger: 日志记录器
            timeout: 超时时间（秒）

        Returns:
            响应文本内容，失败时返回None或状态码
        """
        url_proxy = ProxyConfig.get_config()
        if url_proxy is None:
            logger.warning("Missing proxy config, using direct connection")
            time.sleep(1)  # 无代理时增加延时
            proxies = {}
        else:
            logger.info(f"Using proxy host: {url_proxy.host}")
            proxies = {
                "http": url_proxy.to_str(),
                "https": url_proxy.to_str()
            }

        try:
            logger.info(f"GET {url}")
            response = requests.get(
                url,
                params=params,
                headers=headers,
                cookies=cookies,
                proxies=proxies,
                timeout=timeout
            )
            return HttpProxy._parse_response(url, response, logger)
        except Exception as e:
            logger.warning(f"Proxy request failed: {e}")
            return None

    @staticmethod
    def do_post(
        url: str,
        params: Optional[Dict[str, Any]] = None,
        data: Optional[Dict[str, Any]] = None,
        json: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
        cookies: Optional[Dict[str, str]] = None,
        logger: logging.Logger = logger,
        timeout: int = 1000
    ) -> Optional[Union[str, int]]:
        """
        发送带代理的POST请求

        Args:
            url: 请求URL
            params: 查询参数
            data: 表单数据
            json: JSON数据
            headers: 请求头
            cookies: Cookie
            logger: 日志记录器
            timeout: 超时时间（秒）

        Returns:
            响应文本内容，失败时返回None或状态码
        """
        url_proxy = ProxyConfig.get_config()
        if url_proxy is None:
            logger.warning("Missing proxy config, using direct connection")
            time.sleep(1)  # 无代理时增加延时
            proxies = {}
        else:
            logger.info(f"Using proxy host: {url_proxy.host}")
            proxies = {
                "http": url_proxy.to_str(),
                "https": url_proxy.to_str()
            }

        try:
            logger.info(f"POST {url}")
            response = requests.post(
                url,
                params=params,
                data=data,
                json=json,
                headers=headers,
                cookies=cookies,
                proxies=proxies,
                timeout=timeout
            )
            return HttpProxy._parse_response(url, response, logger)
        except Exception as e:
            logger.warning(f"Proxy request failed: {e}")
            return None

    @staticmethod
    def _parse_response(
        url: str,
        response: requests.Response,
        logger: logging.Logger = logger
    ) -> Optional[Union[str, int]]:
        """
        解析HTTP响应

        Args:
            url: 请求URL
            response: 响应对象
            logger: 日志记录器

        Returns:
            成功时返回响应文本，失败时返回状态码或None
        """
        if response is None:
            return None
        if response.status_code != 200:
            logger.error(f"HTTP error: {url} | {response.status_code} | {response.reason}")
            return response.status_code
        return response.text


if __name__ == '__main__':
    proxy_config_list = ProxyConfig.get_config_list()
    for item in proxy_config_list:
        print(item)



"""
邮件客户端工具模块

提供邮件发送功能，支持：
- QQ邮箱SMTP发送
- HTML格式邮件内容
- 多账户随机轮换
- 文件附件

配置环境变量（pwd为SMTP授权码，非邮箱登录密码）：
    export MAIL_ACCT="{email1}##{pwd1}^^{email2}##{pwd2}"

示例（仅支持QQ邮箱）：
    export MAIL_ACCT="{123@qq.com}##{smtp_pwd_123}^^{456@qq.com}##{smtp_pwd_456}"
"""
import os
import random
import logging
import smtplib
from typing import List, Optional, Union, Set

from email.header import Header
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart

from xsdk.template import HtmlTemplate
from xsdk.file_util import FileUtil
from xsdk.xutil import Xutil

logger = logging.getLogger(__name__)


class MailAccountData:
    """
    邮箱账户配置类

    Attributes:
        user: 用户邮箱地址
        password: SMTP授权码（非邮箱登录密码）
        smtp_server: SMTP服务器地址
        smtp_port: SMTP服务器端口
    """

    def __init__(self, usr: str = "", pwd: str = ""):
        self.user = usr
        self.password = pwd
        self.smtp_server = "smtp.qq.com"
        self.smtp_port = 465

    def __str__(self) -> str:
        return str(self.__dict__)

    @staticmethod
    def get_default_account() -> "MailAccountData":
        """
        从环境变量获取默认邮箱账户（随机选择）

        Returns:
            MailAccountData实例
        """
        config_str = Xutil.get_env_param("MAIL_ACCT")
        if Xutil.is_empty(config_str):
            return MailAccountData()

        account_list = config_str.split("^^")
        if not account_list:
            return MailAccountData()

        default_account_list = []
        for account_info in account_list:
            if "##" not in account_info:
                continue
            usr, pwd = account_info.split("##", 1)
            default_account_list.append(MailAccountData(usr.strip(), pwd.strip()))

        if not default_account_list:
            return MailAccountData()

        idx = random.randint(0, len(default_account_list) - 1)
        return default_account_list[idx]


class MailContentData:
    """
    邮件内容数据类

    Attributes:
        title: 内容标题
        data: 具体内容（字符串、列表或字典列表）
        table_flag: 是否将data生成为HTML表格
    """

    def __init__(self, title: str, data: Union[str, List, Set], table_flag: bool = False):
        """
        Args:
            title: 内容描述
            data: 具体内容
            table_flag: 是否生成表格（如果为True，data必须是字典列表）
        """
        self.title = title
        self.data = data
        self.table_flag = table_flag

    @staticmethod
    def get_mail_content(mail_content_list: Union["MailContentData", List]) -> str:
        """
        将邮件内容数据转换为HTML格式

        Args:
            mail_content_list: MailContentData实例或列表

        Returns:
            HTML格式的邮件内容
        """
        if mail_content_list is None or (isinstance(mail_content_list, list) and len(mail_content_list) == 0):
            return HtmlTemplate.get_html("")

        if isinstance(mail_content_list, MailContentData):
            mail_content_list = [mail_content_list]

        if not isinstance(mail_content_list, list):
            return HtmlTemplate.get_html(str(mail_content_list))

        content = ""
        other_data_list = []

        for mail_content_data in mail_content_list:
            if not isinstance(mail_content_data, MailContentData):
                other_data_list.append(mail_content_data)
                continue

            content_div = ""
            title = f"<b>{mail_content_data.title}:</b>"
            data = mail_content_data.data
            table_flag = mail_content_data.table_flag

            content_div += f"</br>{title}<br>"

            if table_flag:
                table = HtmlTemplate.get_table(data)
                content_div += table
            else:
                if isinstance(data, (list, set)):
                    data_sorted = sorted(data)
                    for line in data_sorted:
                        content_div += f"{line}</br>"
                else:
                    content_div += f"{data}</br>"

            content += f"<div style='clear:both;'>{content_div}</div></br>"

        # 处理非MailContentData类型的数据
        for data in other_data_list:
            if isinstance(data, (set, list, tuple)):
                for line in data:
                    content += f"{line}</br>"
            else:
                content += f"{data}</br>"

        return HtmlTemplate.get_html(content)


class MailClient:
    """
    邮件客户端类

    提供邮件发送功能，支持HTML格式和附件。

    Example:
        >>> content = MailContentData("标题", "内容")
        >>> MailClient.send_mail_v2("邮件主题", "receiver@qq.com", [content])
    """

    @staticmethod
    def get_receiver_list(receiver_list: Union[str, List[str]], suffix: str = "@qq.com") -> List[str]:
        """
        处理收件人列表

        Args:
            receiver_list: 收件人列表（字符串用分号分隔，或列表）
            suffix: 默认邮箱后缀

        Returns:
            处理后的收件人列表
        """
        if isinstance(receiver_list, str):
            receiver_list = receiver_list.split(";")

        result = []
        for receiver in receiver_list:
            if Xutil.is_empty(receiver):
                continue
            receiver = receiver.strip()
            if suffix and not receiver.endswith(suffix) and not receiver.endswith(".com"):
                receiver = f"{receiver}{suffix}"
            result.append(receiver)

        return result

    @staticmethod
    def send_mail_v2(
        mail_title: str,
        mail_to: Union[str, List[str]],
        mail_content_list: Optional[List[MailContentData]] = None,
        attach_file_list: Optional[List[str]] = None,
        logger: logging.Logger = logger
    ) -> bool:
        """
        发送HTML格式邮件

        Args:
            mail_title: 邮件主题
            mail_to: 收件人（字符串或列表）
            mail_content_list: MailContentData列表
            attach_file_list: 附件文件路径列表
            logger: 日志记录器

        Returns:
            发送是否成功
        """
        try:
            mail_content = MailContentData.get_mail_content(mail_content_list)
            return MailClient.send_mail_smtp(mail_title, mail_to, mail_content, attach_file_list, logger)
        except Exception as e:
            logger.warning(f"Failed to send mail: {e}")
            logger.exception(e)
            return False

    @staticmethod
    def send_mail_smtp(
        mail_title: str,
        mail_to: Union[str, List[str]],
        mail_content: str,
        attach_file_list: Optional[List[str]] = None,
        logger: logging.Logger = logger
    ) -> bool:
        """
        使用SMTP发送邮件

        Args:
            mail_title: 邮件主题
            mail_to: 收件人
            mail_content: 邮件内容（HTML格式）
            attach_file_list: 附件文件路径列表
            logger: 日志记录器

        Returns:
            发送是否成功
        """
        receiver_list = MailClient.get_receiver_list(mail_to)

        mail_account = MailAccountData.get_default_account()
        message = MIMEMultipart()
        message['From'] = mail_account.user
        message['To'] = ",".join(receiver_list)
        message['Subject'] = Header(mail_title, 'utf-8')

        # 邮件正文
        message.attach(MIMEText(mail_content, 'HTML', 'utf-8'))

        # 添加附件
        if attach_file_list:
            for attach_file in attach_file_list:
                if os.path.exists(attach_file):
                    attach_file_name = FileUtil.get_file_name(attach_file)
                    with open(attach_file, 'rb') as f:
                        att = MIMEText(f.read(), 'base64', 'utf-8')
                    att["Content-Type"] = 'application/octet-stream'
                    att["Content-Disposition"] = f'attachment; filename={attach_file_name}'
                    message.attach(att)

        try:
            smtp = smtplib.SMTP_SSL(mail_account.smtp_server, mail_account.smtp_port)
            smtp.set_debuglevel(0)
            smtp.ehlo(mail_account.smtp_server)
            smtp.login(mail_account.user, mail_account.password)
            smtp.sendmail(mail_account.user, receiver_list, message.as_string())
            smtp.quit()
            logger.info("Mail sent successfully")
            return True
        except smtplib.SMTPException as e:
            logger.warning("Failed to send mail via SMTP")
            logger.exception(e)
            return False


if __name__ == "__main__":
    test_title = "TEST"
    test_to = "1955463634@qq.com"
    test_content = "TEST CONTENT"
    MailClient.send_mail_smtp(test_title, test_to, test_content, attach_file_list=[])



"""
Redis客户端工具模块

提供Redis缓存操作的封装，支持：
- 键值存取
- 访问计数
- IP限流检查

配置环境变量：
    python3 -m pip install redis
    export REDIS_URL="redis://127.0.0.1:6379/0"
"""
import logging
from typing import Optional, Tuple, Any

import redis

from xsdk.xutil import Xutil

logger = logging.getLogger(__name__)


class RedisClient:
    """
    Redis客户端封装类

    使用单例模式管理Redis连接，提供常用的缓存操作方法。

    Attributes:
        _client: Redis客户端实例（类级别单例）

    Example:
        >>> RedisClient.set_value("name", "test", timeout=3600)
        >>> value = RedisClient.get_value("name")
    """

    _client: Optional[redis.Redis] = None

    @classmethod
    def get_client(cls) -> Optional[redis.Redis]:
        """
        获取Redis客户端实例（单例模式）

        Returns:
            Redis客户端实例，连接失败时返回None
        """
        if cls._client is None:
            try:
                redis_url = Xutil.get_env_param("REDIS_URL")
                cls._client = redis.Redis.from_url(redis_url, decode_responses=True)
            except Exception as e:
                logger.warning(f"Failed to connect to Redis: {e}")
        return cls._client

    @staticmethod
    def set_value(key: str, value: Any, timeout: Optional[int] = None) -> bool:
        """
        设置缓存值

        Args:
            key: 缓存键
            value: 缓存值
            timeout: 过期时间（秒），None表示永不过期

        Returns:
            操作是否成功
        """
        client = RedisClient.get_client()
        if client is None:
            return False
        try:
            client.set(key, value, ex=timeout)
            return True
        except Exception as e:
            logger.warning(f"Failed to set value for key '{key}': {e}")
            return False

    @staticmethod
    def get_value(key: str) -> Optional[str]:
        """
        获取缓存值

        Args:
            key: 缓存键

        Returns:
            缓存值，不存在或异常时返回None
        """
        RedisClient.count(key)
        client = RedisClient.get_client()
        if client is None:
            return None
        try:
            return client.get(key)
        except Exception as e:
            logger.warning(f"Failed to get value for key '{key}': {e}")
            return None

    @staticmethod
    def count(key: str) -> int:
        """
        增加键的访问计数

        Args:
            key: 缓存键

        Returns:
            当前计数值，异常时返回0
        """
        client = RedisClient.get_client()
        if client is None:
            return 0
        try:
            count_key = f"COUNT_{key}"
            return client.incr(count_key)
        except Exception as e:
            logger.warning(f"Failed to increment count for key '{key}': {e}")
            return 0

    @staticmethod
    def check_ip_count(ip: str, max_count: int = 10) -> Tuple[bool, Optional[str]]:
        """
        检查IP访问次数是否超限

        Args:
            ip: IP地址
            max_count: 最大允许访问次数

        Returns:
            (是否允许访问, 错误消息)
            - (True, None): 允许访问
            - (False, 错误消息): 超过限制
        """
        client = RedisClient.get_client()
        if client is None:
            return True, None

        key = f"IP_{ip}"
        try:
            curr_count = client.incr(key)
            if curr_count > max_count:
                return False, f"您（{ip}）已访问超过{max_count}次，请在24小时后重试～"
            return True, None
        except Exception as e:
            logger.warning(f"Failed to check IP count for '{ip}': {e}")
            return True, None

    @staticmethod
    def reset_ip_count(ip: str, default_count: int = 0) -> bool:
        """
        重置IP访问计数

        Args:
            ip: IP地址
            default_count: 重置后的计数值

        Returns:
            操作是否成功
        """
        client = RedisClient.get_client()
        if client is None:
            return False

        key = f"IP_{ip}"
        try:
            client.set(key, default_count)
            return True
        except Exception as e:
            logger.warning(f"Failed to reset IP count for '{ip}': {e}")
            return False


if __name__ == '__main__':
    key = "name"
    value = "test"
    RedisClient.set_value(key, value)
    res = RedisClient.get_value(key)
    print(res)



"""
CSV解析工具模块

提供CSV文件的读写功能，支持：
- 迭代器模式逐行读取
- 自定义字段映射
- 字典或对象列表写入
- 自动处理逗号和引号

Example:
    >>> parser = CsvParser("data.csv", ["name", "age"])
    >>> for row in parser:
    ...     print(row["name"])
    >>> parser.close()
"""
import csv
import logging
from typing import List, Dict, Any, Optional, Tuple, Iterator, IO

logger = logging.getLogger(__name__)


class CsvParser:
    """
    CSV解析器类

    使用迭代器模式逐行读取CSV文件，并将每行映射为字典。

    Attributes:
        file_path: CSV文件路径
        field_list: 字段名列表
        encoding: 文件编码
        file_io: 文件句柄
        file_reader: CSV读取器

    Example:
        >>> fields = ["id", "name", "value"]
        >>> parser = CsvParser("data.csv", fields)
        >>> for row in parser:
        ...     print(row)
        >>> parser.close()

        # 使用上下文管理器
        >>> with CsvParser("data.csv", fields) as parser:
        ...     for row in parser:
        ...         print(row)
    """

    def __init__(
        self,
        file_path: str,
        field_list: List[str],
        encoding: str = "utf-8"
    ):
        """
        初始化CSV解析器

        Args:
            file_path: CSV文件路径
            field_list: 字段名列表，用于将每行数据映射为字典
            encoding: 文件编码，默认utf-8
        """
        self.file_path: str = file_path
        self.field_list: List[str] = field_list
        self.encoding: str = encoding
        self.file_io: Optional[IO[str]] = None
        self.file_reader: Optional[Iterator[List[str]]] = None

    def __iter__(self) -> "CsvParser":
        """
        初始化迭代器

        打开CSV文件并创建读取器。

        Returns:
            CsvParser实例自身
        """
        self.file_io = open(self.file_path, "r", encoding=self.encoding)
        self.file_reader = csv.reader(self.file_io)
        return self

    def __next__(self) -> Dict[str, str]:
        """
        获取下一行数据

        Returns:
            字段名到字段值的字典

        Raises:
            StopIteration: 文件读取完毕时抛出
        """
        line = next(self.file_reader)
        line_dict = dict(zip(self.field_list, line))
        return line_dict

    def close(self) -> None:
        """
        关闭文件句柄

        释放文件资源。
        """
        if self.file_io:
            self.file_io.close()
            self.file_io = None
            self.file_reader = None

    def __enter__(self) -> "CsvParser":
        """
        上下文管理器入口

        Returns:
            CsvParser实例自身
        """
        return self.__iter__()

    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        """
        上下文管理器退出

        Args:
            exc_type: 异常类型
            exc_val: 异常值
            exc_tb: 异常追溯
        """
        self.close()

    @staticmethod
    def _escape_csv_value(value: str) -> str:
        """
        转义CSV字段值

        处理包含逗号或引号的字段值。

        Args:
            value: 原始字段值

        Returns:
            转义后的字段值
        """
        value = str(value).strip()
        if "," in value or '"' in value:
            # 转义内部引号
            value = value.replace('"', '""')
            value = f'"{value}"'
        return value

    @staticmethod
    def write_csv(
        data_list: List[Any],
        csv_file_path: str,
        write_title: bool = True
    ) -> Tuple[bool, str]:
        """
        写入数据到CSV文件

        数据列表中的元素可以是字典或对象（会自动转换为__dict__）。

        Args:
            data_list: 数据列表
            csv_file_path: 输出文件路径
            write_title: 是否写入标题行

        Returns:
            元组 (是否成功, 消息)

        Example:
            >>> data = [{"name": "Alice", "age": 30}, {"name": "Bob", "age": 25}]
            >>> success, msg = CsvParser.write_csv(data, "output.csv")
        """
        if data_list is None or len(data_list) == 0:
            msg = "No data to write!"
            logger.warning(msg)
            return False, msg

        # 转换对象为字典
        converted_data: List[Dict[str, Any]] = []
        for data in data_list:
            if not isinstance(data, dict):
                converted_data.append(data.__dict__)
            else:
                converted_data.append(data)

        with open(csv_file_path, "w", encoding="utf-8", newline="") as file_writer:
            # 获取字段列表
            field_list = list(converted_data[0].keys())

            # 写入标题行
            if write_title:
                title_values = [CsvParser._escape_csv_value(key) for key in field_list]
                file_writer.write(",".join(title_values) + "\n")

            # 写入数据内容
            for line in converted_data:
                line_values = [
                    CsvParser._escape_csv_value(line.get(key, ""))
                    for key in field_list
                ]
                file_writer.write(",".join(line_values) + "\n")

        msg = f"Successfully wrote file: {csv_file_path}"
        logger.info(msg)
        return True, msg


if __name__ == "__main__":
    test_data = [
        {"name": "henry", "job": "dev"},
        {"name": "henry2", "job": "dev2"}
    ]
    CsvParser.write_csv(test_data, "./test_attach.csv", write_title=True)



"""
XSDK - Python通用开发工具包

提供常用的工具类和方法，包括：
- xutil: 通用工具（字符串、日期、JSON处理等）
- template: HTML模板生成
- file_util: 文件操作工具
- config_reader: 配置文件读取
- csv_parser: CSV文件解析
- excel_util: Excel文件处理
- file_parser: 多格式文件解析
- bean_util: 对象序列化工具
- chart_util: 图表生成工具
- mysql_client: MySQL数据库客户端
- redis_client: Redis缓存客户端
- http_client: HTTP客户端
- http_proxy: HTTP代理工具
- spider_util: 网页爬虫工具
- mail_client: 邮件发送客户端
- cos_client: 腾讯云COS存储客户端
- xlog: 日志管理
"""

__version__ = "1.0.6"
__author__ = "xsdk"

# 基础工具模块（无外部依赖）
from xsdk.xutil import Xutil
from xsdk.template import HtmlTemplate
from xsdk.file_util import FileUtil
from xsdk.config_reader import ConfigReader

# 导出主要类
__all__ = [
    # 版本信息
    "__version__",
    "__author__",
    # 基础工具
    "Xutil",
    "HtmlTemplate",
    "FileUtil",
    "ConfigReader",
]



"""
配置文件读取工具模块

解析系统配置文件（INI格式），支持：
- INI文件解析
- 多Section配置
- 类型转换（字符串、整数、浮点数、字典）
- 环境判断（开发/测试/生产）

注意：底层基础工具，禁止引入过多依赖

Example:
    >>> reader = ConfigReader.get_instance("config.ini")
    >>> db_host = reader.get("database", "host")
    >>> db_port = reader.get_int("database", "port")
"""
import os
import json
import configparser
from typing import Dict, Any, Optional


class ConfigError(Exception):
    """配置文件错误异常"""
    pass


class ConfigUtil:
    """
    配置工具类

    提供静态方法进行配置文件路径处理和读取。

    Example:
        >>> sys_path = ConfigUtil.get_sys_path(__file__)
        >>> conf_dict = ConfigUtil.read_conf("config.ini")
    """

    @staticmethod
    def get_sys_path(py_file: str, sys_name: Optional[str] = None) -> str:
        """
        获取系统根路径

        根据Python文件路径推断系统根目录。

        Args:
            py_file: Python文件路径（通常传入__file__）
            sys_name: 系统名称（可选）

        Returns:
            系统根目录路径
        """
        file_path = str(os.path.abspath(py_file))
        idx_sys_name = -1

        if sys_name and len(str(sys_name)) > 0:
            idx_sys_name = file_path.rfind(sys_name)

        idx_src = file_path.rfind("/src/")
        idx_tests = file_path.rfind("/tests/")

        if idx_sys_name > 0:
            sys_path = file_path[0:idx_sys_name + len(sys_name)]
        elif idx_src > 0:
            sys_path = file_path[0:idx_src]
        elif idx_tests > 0:
            sys_path = file_path[0:idx_tests]
        else:
            sys_path = "../"

        return sys_path

    @staticmethod
    def get_env(
        conf_file: str,
        conf_section: str = "sys_info",
        conf_key: str = "sys_env"
    ) -> str:
        """
        获取系统环境配置

        Args:
            conf_file: 配置文件路径
            conf_section: Section名称
            conf_key: 配置键名

        Returns:
            环境标识字符串
        """
        config_reader = ConfigReader.get_instance(conf_file)
        return config_reader.get(conf_section, conf_key)

    @staticmethod
    def is_dev(
        conf_file: str,
        conf_section: str = "sys_info",
        conf_key: str = "sys_env"
    ) -> bool:
        """
        判断是否为开发环境

        Args:
            conf_file: 配置文件路径
            conf_section: Section名称
            conf_key: 配置键名

        Returns:
            是否为开发环境
        """
        return ConfigUtil.get_env(conf_file, conf_section, conf_key) == "DEV"

    @staticmethod
    def is_test(
        conf_file: str,
        conf_section: str = "sys_info",
        conf_key: str = "sys_env"
    ) -> bool:
        """
        判断是否为测试环境

        Args:
            conf_file: 配置文件路径
            conf_section: Section名称
            conf_key: 配置键名

        Returns:
            是否为测试环境
        """
        return ConfigUtil.get_env(conf_file, conf_section, conf_key) == "TEST"

    @staticmethod
    def is_prod(
        conf_file: str,
        conf_section: str = "sys_info",
        conf_key: str = "sys_env"
    ) -> bool:
        """
        判断是否为生产环境

        Args:
            conf_file: 配置文件路径
            conf_section: Section名称
            conf_key: 配置键名

        Returns:
            是否为生产环境
        """
        return ConfigUtil.get_env(conf_file, conf_section, conf_key).startswith("PROD")

    @staticmethod
    def read_conf(conf_file: str) -> Dict[str, Dict[str, str]]:
        """
        读取配置文件

        Args:
            conf_file: 配置文件路径

        Returns:
            嵌套字典结构：{section: {key: value}}

        Raises:
            ConfigError: 配置文件不存在时抛出

        Example:
            >>> conf = ConfigUtil.read_conf("config.ini")
            >>> print(conf["database"]["host"])
        """
        if not os.path.exists(conf_file) or not os.path.isfile(conf_file):
            raise ConfigError(f"Config file not exist: {conf_file}")

        conf_parse = configparser.ConfigParser()
        conf_parse.read(conf_file, encoding="utf-8")

        conf_dict: Dict[str, Dict[str, str]] = {}
        conf_sections = conf_parse.sections()

        for section in conf_sections:
            section_dict: Dict[str, str] = {}
            keys = conf_parse.options(section)
            for key in keys:
                section_dict[key] = conf_parse.get(section, key)
            conf_dict[section] = section_dict

        return conf_dict


class ConfigReader:
    """
    配置文件读取器类

    封装配置字典，提供便捷的配置值获取方法。

    Attributes:
        conf_dict: 配置字典

    Example:
        >>> reader = ConfigReader.get_instance("config.ini")
        >>> host = reader.get("database", "host")
        >>> port = reader.get_int("database", "port")
    """

    def __init__(self, conf_dict: Dict[str, Dict[str, str]]):
        """
        初始化配置读取器

        Args:
            conf_dict: 配置字典
        """
        self.conf_dict: Dict[str, Dict[str, str]] = conf_dict

    @staticmethod
    def get_instance(conf_file: str) -> "ConfigReader":
        """
        创建ConfigReader实例

        Args:
            conf_file: 配置文件路径

        Returns:
            ConfigReader实例
        """
        conf_dict = ConfigUtil.read_conf(conf_file)
        return ConfigReader(conf_dict)

    def get(self, section: str, key: str, default: str = "") -> str:
        """
        获取字符串配置值

        Args:
            section: Section名称
            key: 配置键名
            default: 默认值

        Returns:
            配置值字符串
        """
        if section in self.conf_dict:
            section_dict = self.conf_dict[section]
            if key in section_dict:
                return section_dict[key]
        return default

    def get_int(self, section: str, key: str, default: int = 0) -> int:
        """
        获取整数配置值

        Args:
            section: Section名称
            key: 配置键名
            default: 默认值

        Returns:
            配置值整数
        """
        if section in self.conf_dict:
            section_dict = self.conf_dict[section]
            if key in section_dict:
                return int(section_dict[key])
        return default

    def get_float(self, section: str, key: str, default: float = 0.0) -> float:
        """
        获取浮点数配置值

        Args:
            section: Section名称
            key: 配置键名
            default: 默认值

        Returns:
            配置值浮点数
        """
        if section in self.conf_dict:
            section_dict = self.conf_dict[section]
            if key in section_dict:
                return float(section_dict[key])
        return default

    def get_dict(self, section: str, key: str) -> Dict[str, Any]:
        """
        获取字典配置值

        配置值应为JSON格式字符串。

        Args:
            section: Section名称
            key: 配置键名

        Returns:
            解析后的字典
        """
        if section in self.conf_dict:
            section_dict = self.conf_dict[section]
            if key in section_dict:
                value = section_dict[key]
                return json.loads(value)
        return {}

    def get_section_dict(self, section: str) -> Dict[str, str]:
        """
        获取整个Section的配置字典

        Args:
            section: Section名称

        Returns:
            Section配置字典
        """
        if section in self.conf_dict:
            return self.conf_dict[section]
        return {}

    def get_env(
        self,
        conf_section: str = "sys_info",
        conf_key: str = "sys_env"
    ) -> str:
        """
        获取系统环境配置

        Args:
            conf_section: Section名称
            conf_key: 配置键名

        Returns:
            环境标识字符串
        """
        return self.get(conf_section, conf_key)

    def is_dev(
        self,
        conf_section: str = "sys_info",
        conf_key: str = "sys_env"
    ) -> bool:
        """
        判断是否为开发环境

        Args:
            conf_section: Section名称
            conf_key: 配置键名

        Returns:
            是否为开发环境
        """
        return self.get_env(conf_section, conf_key) == "DEV"

    def is_test(
        self,
        conf_section: str = "sys_info",
        conf_key: str = "sys_env"
    ) -> bool:
        """
        判断是否为测试环境

        Args:
            conf_section: Section名称
            conf_key: 配置键名

        Returns:
            是否为测试环境
        """
        return self.get_env(conf_section, conf_key) == "TEST"

    def is_prod(
        self,
        conf_section: str = "sys_info",
        conf_key: str = "sys_env"
    ) -> bool:
        """
        判断是否为生产环境

        Args:
            conf_section: Section名称
            conf_key: 配置键名

        Returns:
            是否为生产环境
        """
        return self.get_env(conf_section, conf_key).startswith("PROD")


if __name__ == "__main__":
    pass



"""
通用工具模块

提供常用的工具方法，包括：
- 字符串处理（MD5、清理、简化等）
- 日期时间处理（计算、格式化、区间等）
- 数字处理（解析、格式化）
- JSON/字典操作
- 环境变量读取
"""
import os
import ast
import json
import time
import datetime
from hashlib import md5
from typing import Optional, List, Dict, Any, Union

import arrow


class Xutil:
    """
    通用工具类

    提供静态方法进行常用的数据处理操作。

    Example:
        >>> Xutil.md5_str("hello")
        '5d41402abc4b2a76b9719d911017c592'
        >>> Xutil.get_date_current()
        '2024-01-01 12:00:00'
    """

    # ==================== 字符串处理 ====================

    @staticmethod
    def md5_str(str_input: Any, case: str = "LOWER") -> str:
        """
        计算字符串的MD5值

        Args:
            str_input: 输入字符串
            case: 结果大小写（LOWER/UPPER）

        Returns:
            MD5哈希字符串
        """
        ret = md5(str(str_input).strip().encode("utf-8")).hexdigest()
        if case == "UPPER":
            return ret.upper()
        elif case == "LOWER":
            return ret.lower()
        return ret

    @staticmethod
    def clean_str(str_input: Any, case: str = "LOWER") -> str:
        """
        清除字符串中的空格

        Args:
            str_input: 输入字符串
            case: 结果大小写（LOWER/UPPER/其他保持原样）

        Returns:
            去空格后的字符串
        """
        ret = str_input if str_input is not None else ""
        ret = str(ret).replace(" ", "").strip()
        if case == "UPPER":
            return ret.upper()
        elif case == "LOWER":
            return ret.lower()
        return ret

    @staticmethod
    def simple_str(str_input: Any) -> str:
        """
        简化字符串：移除换行、首尾空格，合并相邻空格

        Args:
            str_input: 输入字符串

        Returns:
            简化后的字符串
        """
        if str_input is None:
            return ""
        str_simple = str(str_input).replace("\n", " ").strip()
        while "  " in str_simple:
            str_simple = str_simple.replace("  ", " ")
        return str_simple

    @staticmethod
    def link_words(*words, link_str: str = " | ") -> str:
        """
        连接多个词语

        Args:
            *words: 要连接的词语
            link_str: 连接符

        Returns:
            连接后的字符串
        """
        word_list = [str(item) for item in words]
        return link_str.join(word_list)

    @staticmethod
    def split(str_input: str, link_str: str = " | ") -> List[str]:
        """
        按分隔符分割字符串

        Args:
            str_input: 输入字符串
            link_str: 分隔符

        Returns:
            分割后的列表
        """
        return str_input.split(link_str)

    @staticmethod
    def contains_chinese(str_input: Any) -> bool:
        """
        判断字符串是否包含中文

        Args:
            str_input: 输入字符串

        Returns:
            是否包含中文
        """
        if str_input and isinstance(str_input, str):
            for char in str_input:
                if '\u4e00' <= char <= '\u9fa5':
                    return True
        return False

    @staticmethod
    def recode_str(str_input: str, code_ori: str = "latin1", code_new: str = "gbk") -> str:
        """
        重新编码字符串（解决中文乱码问题）

        Args:
            str_input: 输入字符串
            code_ori: 原始编码
            code_new: 目标编码

        Returns:
            重新编码后的字符串
        """
        result = None
        try:
            result = str_input.encode(code_ori, 'ignore').decode(code_new, 'ignore')
        except UnicodeEncodeError as error:
            print(f"Failed to recode string: {str_input} | {error}")

        if result is None or result.strip() == "":
            result = str_input
        return result

    # ==================== 日期时间处理 ====================

    @staticmethod
    def count_years(date1: str, date2: str) -> float:
        """
        计算两个日期之间的年数

        Args:
            date1: 日期1
            date2: 日期2

        Returns:
            相差的年数
        """
        days = Xutil.count_days(date1, date2)
        return abs(days) / 365

    @staticmethod
    def count_days(begin_date: str, end_date: str) -> int:
        """
        计算两个日期之间的天数

        Args:
            begin_date: 开始日期
            end_date: 结束日期

        Returns:
            相差的天数
        """
        date_format = "%Y%m%d"
        date1 = str(begin_date).replace("-", "")[0:8]
        date2 = str(end_date).replace("-", "")[0:8]
        try:
            date1 = datetime.datetime.strptime(date1, date_format)
            date2 = datetime.datetime.strptime(date2, date_format)
        except ValueError:
            print("日期格式不正确，请输入正确的 YYYYMMDD 格式日期。")
            return 0
        delta = date2 - date1
        return delta.days

    @staticmethod
    def format_date(
        date_input: str,
        format_input: str = "%m %d %Y",
        format_output: str = "%Y-%m-%d"
    ) -> str:
        """
        格式化日期字符串

        Args:
            date_input: 输入日期
            format_input: 输入格式
            format_output: 输出格式

        Returns:
            格式化后的日期字符串
        """
        _date = datetime.datetime.strptime(str(date_input).strip(), format_input)
        return _date.strftime(format_output)

    @staticmethod
    def format_datetime(datetime_input: Any, format_output: str = "YYYY-MM-DD HH:mm:ss") -> str:
        """
        格式化日期时间

        Args:
            datetime_input: 输入日期时间
            format_output: 输出格式（arrow格式）

        Returns:
            格式化后的日期时间字符串
        """
        if not datetime_input:
            return ""
        return arrow.get(datetime_input).format(format_output)

    @staticmethod
    def format_timestamp(timestamp: int, format_output: str = "%Y-%m-%d") -> str:
        """
        格式化时间戳

        Args:
            timestamp: 时间戳
            format_output: 输出格式

        Returns:
            格式化后的日期字符串
        """
        timestamp = time.localtime(int(timestamp))
        return time.strftime(format_output, timestamp)

    @staticmethod
    def date2timestamp(date_str: str) -> int:
        """
        日期字符串转时间戳

        Args:
            date_str: 日期字符串（YYYYMMDD或YYYY-MM-DD）

        Returns:
            时间戳
        """
        date_str = date_str.replace("-", "")[0:8]
        date_obj = datetime.datetime.strptime(date_str, '%Y%m%d')
        return int(date_obj.timestamp())

    @staticmethod
    def get_date_difference(date_begin: str, date_end: str) -> int:
        """
        计算两个日期相隔天数

        Args:
            date_begin: 开始日期
            date_end: 结束日期

        Returns:
            相差天数
        """
        date_begin = date_begin.replace("-", "").replace("/", "")[0:8]
        date_end = date_end.replace("-", "").replace("/", "")[0:8]

        date_begin = time.strptime(date_begin, "%Y%m%d")
        date_end = time.strptime(date_end, "%Y%m%d")

        date_begin = datetime.datetime(date_begin[0], date_begin[1], date_begin[2])
        date_end = datetime.datetime(date_end[0], date_end[1], date_end[2])

        return (date_end - date_begin).days

    @staticmethod
    def get_date_current(date_format: str = "%Y-%m-%d %H:%M:%S") -> str:
        """
        获取当前日期时间

        Args:
            date_format: 日期格式

        Returns:
            当前日期时间字符串
        """
        return time.strftime(date_format, time.localtime(time.time()))

    @staticmethod
    def get_date_before(
        date_input: Optional[str] = None,
        before_count: int = 1,
        date_format: str = "%Y-%m-%d"
    ) -> str:
        """
        获取指定日期的前N天

        Args:
            date_input: 指定日期，None取当天
            before_count: 前几天（正数前移，负数后移）
            date_format: 日期格式

        Returns:
            计算后的日期字符串
        """
        if date_input is None:
            date_base = datetime.datetime.now()
        else:
            date_base = datetime.datetime.strptime(date_input, date_format)

        offset = datetime.timedelta(days=-before_count)
        return (date_base + offset).strftime(date_format)

    @staticmethod
    def get_date_next(
        date_input: Optional[str] = None,
        next_count: int = 1,
        date_format: str = "%Y-%m-%d"
    ) -> str:
        """
        获取指定日期的后N天

        Args:
            date_input: 指定日期，None取当天
            next_count: 后几天
            date_format: 日期格式

        Returns:
            计算后的日期字符串
        """
        if date_input is None:
            date_base = datetime.datetime.now()
        else:
            date_base = datetime.datetime.strptime(date_input, date_format)

        offset = datetime.timedelta(days=next_count)
        return (date_base + offset).strftime(date_format)

    @staticmethod
    def get_date_between(
        date_begin: str,
        date_end: str,
        date_format: str = "%Y-%m-%d"
    ) -> List[str]:
        """
        获取日期区间内的所有日期

        Args:
            date_begin: 开始日期
            date_end: 结束日期
            date_format: 日期格式

        Returns:
            日期列表
        """
        if Xutil.is_empty(date_end):
            date_end = date_begin

        date_list = []
        date_begin = datetime.datetime.strptime(date_begin, date_format)
        date_end = datetime.datetime.strptime(date_end, date_format)
        while date_begin <= date_end:
            date_list.append(date_begin.strftime(date_format))
            date_begin += datetime.timedelta(days=1)
        return date_list

    @staticmethod
    def get_month_last_day(month_input: str) -> str:
        """
        获取指定月份的最后一天

        Args:
            month_input: 月份（YYYY-MM）

        Returns:
            该月最后一天的日期
        """
        if len(str(month_input)) == 6:
            month_input = f"{month_input[0:4]}-{month_input[4:6]}"
        month_next = Xutil.get_month_next(month_input)
        date_temp = f"{month_next}-01"
        return Xutil.get_date_before(date_temp)

    @staticmethod
    def get_month_date_first_and_last(
        month_input: str,
        date_format: str = "%Y-%m-%d"
    ) -> tuple:
        """
        获取指定月份的第一天和最后一天

        Args:
            month_input: 月份
            date_format: 日期格式

        Returns:
            (第一天, 最后一天)
        """
        month_input = month_input.replace("-", "")
        year = month_input[0:4]
        month = month_input[4:6]
        return Xutil.get_month_date(year, month, date_format)

    @staticmethod
    def get_month_date(year: Any, month: Any, date_format: str = "%Y-%m-%d") -> tuple:
        """
        获取指定年月的第一天和最后一天

        Args:
            year: 年份
            month: 月份
            date_format: 日期格式

        Returns:
            (第一天, 最后一天)
        """
        year = int(year)
        month = int(month)

        date_first = datetime.datetime(year, month, 1).strftime(date_format)
        if month == 12:
            date_last = datetime.datetime(year, month, 31).strftime(date_format)
        else:
            date_last = (datetime.datetime(year, month + 1, 1) - datetime.timedelta(days=1)).strftime(date_format)
        return date_first, date_last

    @staticmethod
    def get_month_date_list(month_input: str, date_format: str = "%Y-%m-%d") -> List[str]:
        """
        获取指定月份的所有日期

        Args:
            month_input: 月份
            date_format: 日期格式

        Returns:
            日期列表
        """
        date_first, date_last = Xutil.get_month_date_first_and_last(month_input, date_format)
        return Xutil.get_date_between(date_first, date_last, date_format)

    @staticmethod
    def get_month_before(
        month_input: Optional[str] = None,
        before_count: int = 1,
        month_format: str = "YYYY-MM"
    ) -> str:
        """
        获取指定月份的前N月

        Args:
            month_input: 指定月份，None取当月
            before_count: 前几月
            month_format: 月份格式

        Returns:
            计算后的月份
        """
        if not month_input:
            month_input = arrow.now().format(month_format)

        a = arrow.get(month_input, month_format)
        b = a.shift(months=-before_count)
        return b.format(month_format)

    @staticmethod
    def get_month_between(
        month_begin: str,
        month_end: str,
        month_format: str = "YYYY-MM"
    ) -> List[str]:
        """
        获取月份区间内的所有月份

        Args:
            month_begin: 开始月份
            month_end: 结束月份
            month_format: 月份格式

        Returns:
            月份列表
        """
        month_list = []
        while month_begin <= month_end:
            month_list.append(month_begin)
            a = arrow.get(month_begin, month_format)
            b = a.shift(months=1)
            month_begin = b.format(month_format)
        return month_list

    @staticmethod
    def get_month_next(
        month_input: Optional[str] = None,
        next_count: int = 1,
        month_format: str = "YYYY-MM"
    ) -> str:
        """
        获取指定月份的后N月

        Args:
            month_input: 指定月份，None取当月
            next_count: 后几月
            month_format: 月份格式

        Returns:
            计算后的月份
        """
        if not month_input:
            month_input = arrow.now().format(month_format)

        a = arrow.get(month_input, month_format)
        b = a.shift(months=next_count)
        return b.format(month_format)

    @staticmethod
    def get_quarter_last_date(data_date: str) -> str:
        """
        获取指定日期所在季度的最后一天

        Args:
            data_date: 日期（YYYY-MM-DD）

        Returns:
            季度最后一天
        """
        date = datetime.datetime.strptime(data_date, '%Y-%m-%d')
        year = date.year
        month = date.month

        if month <= 3:
            last_day = datetime.datetime(year, 3, 31)
        elif month <= 6:
            last_day = datetime.datetime(year, 6, 30)
        elif month <= 9:
            last_day = datetime.datetime(year, 9, 30)
        else:
            last_day = datetime.datetime(year, 12, 31)

        return last_day.strftime('%Y-%m-%d')

    @staticmethod
    def get_quarter_month_list(year: Any) -> List[str]:
        """
        获取指定年份的季度末月份列表

        Args:
            year: 年份

        Returns:
            季度末月份列表
        """
        year = str(year).strip()
        return [f"{year}-{month}" for month in ["03", "06", "09", "12"]]

    @staticmethod
    def get_quarter_date_list(year: Any) -> List[str]:
        """
        获取指定年份的季度末日期列表

        Args:
            year: 年份

        Returns:
            季度末日期列表
        """
        month_list = Xutil.get_quarter_month_list(year)
        return [Xutil.get_month_last_day(month) for month in month_list]

    @staticmethod
    def get_quarter_date_list_in_latest_years(
        year_count: int = 20,
        year_end: Optional[int] = None
    ) -> List[str]:
        """
        获取最近N年的季度末日期列表

        Args:
            year_count: 年数
            year_end: 结束年份

        Returns:
            季度末日期列表（降序）
        """
        date_list = []
        if year_end and year_end > 0:
            year_curr = int(year_end)
        else:
            year_curr = int(Xutil.get_date_before()[0:4])

        for i in range(year_count):
            year = year_curr - i
            date_list.extend(Xutil.get_quarter_date_list(year))

        date_current = Xutil.get_date_before()
        date_list = [item for item in date_list if item < date_current]
        date_list.sort(reverse=True)
        return date_list

    @staticmethod
    def get_quarter_date_list_in_years(
        year: Optional[int] = None,
        year_count: int = 3
    ) -> List[str]:
        """
        获取指定年份开始的N年季度末日期列表

        Args:
            year: 起始年份
            year_count: 年数

        Returns:
            季度末日期列表（降序）
        """
        date_list = []
        if year and year > 0:
            year_curr = int(year)
        else:
            year_curr = int(Xutil.get_date_before()[0:4])

        for i in range(year_count):
            y = year_curr - i
            date_list.extend(Xutil.get_quarter_date_list(y))

        date_current = Xutil.get_date_before()
        date_list = [item for item in date_list if item < date_current]
        date_list.sort(reverse=True)
        return date_list

    @staticmethod
    def get_default_sett_month() -> str:
        """获取默认结算月份（4天前所在月份）"""
        return Xutil.get_date_before(before_count=4)[0:7]

    @staticmethod
    def is_valid_date(date_input: str, date_format: str = "%Y-%m-%d") -> bool:
        """
        判断是否为有效日期

        Args:
            date_input: 日期字符串
            date_format: 日期格式

        Returns:
            是否有效
        """
        try:
            time.strptime(date_input, date_format)
            return True
        except (TypeError, ValueError):
            return False

    @staticmethod
    def is_invalid_date(date_input: str, date_format: str = "%Y-%m-%d") -> bool:
        """
        判断是否为无效日期

        Args:
            date_input: 日期字符串
            date_format: 日期格式

        Returns:
            是否无效
        """
        return not Xutil.is_valid_date(date_input, date_format)

    # ==================== 数字处理 ====================

    @staticmethod
    def format_float(num_input: Any, accuracy: int = 4) -> str:
        """
        格式化浮点数精度

        Args:
            num_input: 输入数字
            accuracy: 小数位数

        Returns:
            格式化后的字符串
        """
        if num_input in [None, "", "-"]:
            num_input = 0
        temp = str(num_input).replace(",", "").replace(" ", "")
        if "%" in temp:
            temp = str(float(temp.replace("%", "")) / 100)
        return f"%.{accuracy}f" % Xutil.parse_to_float(temp)

    @staticmethod
    def parse_to_int(num_input: Any, ignore_err: bool = True) -> int:
        """
        解析为整数

        Args:
            num_input: 输入值
            ignore_err: 是否忽略错误

        Returns:
            整数值
        """
        result = 0
        if Xutil.is_empty(num_input):
            return result
        try:
            result = int(float(num_input))
        except Exception as err:
            if not ignore_err:
                raise err
        return result

    @staticmethod
    def parse_to_float(num_input: Any, ignore_error: bool = True) -> float:
        """
        解析为浮点数

        Args:
            num_input: 输入值
            ignore_error: 是否忽略错误

        Returns:
            浮点数值
        """
        result = 0.0
        try:
            result = float(num_input)
        except Exception as err:
            if not ignore_error:
                raise err
        return result

    # ==================== JSON/字典操作 ====================

    @staticmethod
    def parse_to_json(str_input: str) -> Dict:
        """
        字符串转JSON对象

        Args:
            str_input: JSON字符串

        Returns:
            解析后的字典
        """
        try:
            res: dict = json.loads(str_input)
        except Exception:
            res: dict = ast.literal_eval(str_input)
        return res

    @staticmethod
    def parse_to_dict(
        input_str: str,
        split_outer: str = "&",
        split_inner: str = "="
    ) -> Dict[str, str]:
        """
        解析键值对字符串为字典

        Args:
            input_str: 输入字符串（如k1=v1&k2=v2）
            split_outer: 外层分隔符
            split_inner: 内层分隔符

        Returns:
            解析后的字典
        """
        if not input_str:
            return {}

        result_dict = {}
        for item in input_str.split(split_outer):
            idx = item.find(split_inner)
            if idx > 0:
                key = item[0:idx]
                value = item[idx + 1:]
                result_dict[key] = value
        return result_dict

    @staticmethod
    def get_from_dict(input_dict: Dict, key: str, default: Any = '') -> Any:
        """
        从字典安全获取值

        Args:
            input_dict: 输入字典
            key: 键名
            default: 默认值

        Returns:
            键对应的值或默认值
        """
        if not input_dict or key not in input_dict:
            return default
        result = input_dict[key]
        return default if result is None else result

    @staticmethod
    def get_float_from_dict(input_dict: Dict, key: str, default: float = 0.0) -> float:
        """
        从字典获取浮点数

        Args:
            input_dict: 输入字典
            key: 键名
            default: 默认值

        Returns:
            浮点数值
        """
        if Xutil.is_empty(input_dict) or key not in input_dict:
            return default
        value = input_dict[key]
        if Xutil.is_empty(value):
            return default
        return float(value)

    @staticmethod
    def get_int_from_dict(input_dict: Dict, key: str, default: int = 0) -> int:
        """
        从字典获取整数

        Args:
            input_dict: 输入字典
            key: 键名
            default: 默认值

        Returns:
            整数值
        """
        return int(Xutil.get_float_from_dict(input_dict, key, float(default)))

    # ==================== 通用判断 ====================

    @staticmethod
    def is_empty(value: Any) -> bool:
        """
        判断值是否为空

        Args:
            value: 输入值

        Returns:
            是否为空
        """
        if value is None:
            return True
        if str(value).strip() == "":
            return True
        if isinstance(value, (set, list, dict, tuple)) and len(value) == 0:
            return True
        return False

    @staticmethod
    def is_not_empty(value: Any) -> bool:
        """
        判断值是否非空

        Args:
            value: 输入值

        Returns:
            是否非空
        """
        return not Xutil.is_empty(value)

    @staticmethod
    def get_env_param(key: str) -> str:
        """
        获取环境变量

        Args:
            key: 环境变量名

        Returns:
            环境变量值
        """
        res = ""
        try:
            res = os.environ.get(key) or ""
        except Exception as err:
            print(err)
        return res


if __name__ == '__main__':
    print(Xutil.format_float(1, 4))



"""
Bean工具模块

提供对象序列化和反序列化功能，支持：
- 字典转对象（Bean）
- 对象列表批量转换
- 对象转字典
- 基础Bean类

Example:
    >>> class User(BaseBean):
    ...     def __init__(self):
    ...         self.name = ""
    ...         self.age = 0
    >>> data = {"name": "Alice", "age": 30}
    >>> user = BeanUtil.parse2bean(data, User)
"""
import json
import logging
from typing import Any, Dict, List, Optional, Type, TypeVar

logger = logging.getLogger(__name__)

# 泛型类型变量
T = TypeVar("T")


class BeanError(Exception):
    """
    Bean操作异常类

    Attributes:
        code: 错误代码
        message: 错误消息
    """

    def __init__(self, code: int = -100, message: str = "BEAN_ERROR"):
        """
        初始化异常

        Args:
            code: 错误代码
            message: 错误消息
        """
        super().__init__(message)
        self.code = code
        self.message = message

    def __str__(self) -> str:
        """
        返回异常的字符串表示

        Returns:
            JSON格式的字符串
        """
        return json.dumps({"code": self.code, "message": self.message})


class BeanUtil:
    """
    Bean工具类

    提供静态方法进行对象和字典之间的转换。

    Example:
        >>> class Person:
        ...     def __init__(self):
        ...         self.name = ""
        >>> data = {"name": "Alice", "extra": "ignored"}
        >>> person = BeanUtil.parse2bean(data, Person)
        >>> print(person.name)  # Alice
    """

    @staticmethod
    def parse2bean(data: Optional[Dict[str, Any]], cls: Type[T]) -> Optional[T]:
        """
        将字典转换为指定类型的对象

        只有对象定义的属性才会被设置，字典中的额外字段会被忽略。

        Args:
            data: 源数据字典
            cls: 目标类类型

        Returns:
            转换后的对象实例，data为None时返回None

        Raises:
            BeanError: data不是字典类型时抛出
        """
        if data is None or isinstance(data, cls):
            return data

        if not isinstance(data, dict):
            raise BeanError(message=f"Fail to parse bean! {data}")

        bean_data = cls()
        bean_keys = bean_data.__dict__.keys()

        bean_dict = {}
        for key in bean_keys:
            if key in data:
                bean_dict[key] = data[key]

        bean_data.__dict__.update(bean_dict)
        return bean_data

    @staticmethod
    def parse2bean_list(
        data_list: Optional[List[Dict[str, Any]]],
        cls: Type[T]
    ) -> Optional[List[T]]:
        """
        将字典列表批量转换为对象列表

        Args:
            data_list: 源数据字典列表
            cls: 目标类类型

        Returns:
            转换后的对象列表，data_list为空时返回None

        Raises:
            BeanError: data_list不是列表类型时抛出
        """
        if data_list is None or len(data_list) == 0:
            return None

        if not isinstance(data_list, list):
            raise BeanError(message=f"Fail to parse bean list! {data_list}")

        bean_list: List[T] = []
        for data in data_list:
            bean_data = BeanUtil.parse2bean(data, cls)
            if bean_data is not None:
                bean_list.append(bean_data)

        return bean_list

    @staticmethod
    def parse_to_dict_list(data_list: Optional[List[Any]]) -> List[Dict[str, Any]]:
        """
        将对象列表转换为字典列表

        Args:
            data_list: 对象列表

        Returns:
            字典列表
        """
        if data_list is None or len(data_list) == 0:
            return []

        result_list: List[Dict[str, Any]] = []
        for line in data_list:
            if hasattr(line, "__dict__"):
                line = line.__dict__
            result_list.append(line)

        return result_list


class BaseBean:
    """
    基础Bean类

    提供通用的序列化和反序列化功能，可作为数据类的基类。

    Example:
        >>> class User(BaseBean):
        ...     def __init__(self):
        ...         self.name = ""
        ...         self.age = 0
        >>> user = User()
        >>> user.decode({"name": "Alice", "age": 30})
        >>> print(user)  # {"name": "Alice", "age": 30}
    """

    def __str__(self) -> str:
        """
        返回对象的字符串表示

        Returns:
            JSON格式的字符串
        """
        return json.dumps(self.__dict__, ensure_ascii=False, default=str)

    def to_dict(self) -> Dict[str, Any]:
        """
        将对象转换为字典

        Returns:
            对象属性字典
        """
        return self.__dict__.copy()

    def decode(self, data: Any) -> "BaseBean":
        """
        从数据源解码并更新对象属性

        Args:
            data: 数据源，可以是字典、JSON字符串或BaseBean对象

        Returns:
            更新后的对象自身

        Raises:
            BeanError: 数据格式无效时抛出
        """
        if isinstance(data, BaseBean):
            data = data.__dict__
        elif isinstance(data, str):
            try:
                data = json.loads(data)
            except json.JSONDecodeError:
                # 尝试替换单引号后解析
                data = json.loads(str(data).replace("'", '"'))

        if not isinstance(data, dict):
            raise BeanError(message=f"Fail to decode! {data}")

        bean_dict: Dict[str, Any] = {}
        bean_keys = self.__dict__.keys()
        for key in bean_keys:
            if key in data:
                bean_dict[key] = data[key]

        self.__dict__.update(bean_dict)
        return self



"""
文件解析工具模块

提供多种格式的文件解析功能，支持：
- 纯文本文件（TEXT）
- GZIP压缩文件
- ZIP压缩文件
- BZ2压缩文件
- TGZ压缩包（tar.gz）

使用迭代器模式逐行读取文件，将每行解析为字典格式。

Example:
    >>> field_list = ["name", "age", "city"]
    >>> parser = FileParser("data.txt", field_list, ",")
    >>> for row in parser:
    ...     print(row["name"], row["age"])
"""
import os
import bz2
import gzip
import queue
import tarfile
import zipfile
import logging
from typing import List, Dict, Optional, Any, Iterator, Union, IO

logger = logging.getLogger(__name__)


class FileType:
    """
    文件类型常量类

    定义支持的文件类型标识符。

    Attributes:
        FILE_TYPE_TEXT: 纯文本文件
        FILE_TYPE_GZIP: GZIP压缩文件（.gz）
        FILE_TYPE_ZIP: ZIP压缩文件（.zip）
        FILE_TYPE_BZ2: BZ2压缩文件（.bz2）
        FILE_TYPE_TGZ: TGZ压缩包（.tar.gz）
    """
    FILE_TYPE_TEXT: int = 1
    FILE_TYPE_GZIP: int = 2
    FILE_TYPE_ZIP: int = 3
    FILE_TYPE_BZ2: int = 4
    FILE_TYPE_TGZ: int = 5


class FileParser:
    """
    文件解析器类

    使用迭代器模式逐行读取文件，并将每行按分隔符解析为字典。
    支持多种文件格式：纯文本、GZIP、ZIP、BZ2、TGZ。

    Attributes:
        file_name: 文件路径
        filed_list: 字段名列表
        filed_sep: 字段分隔符
        file_type: 文件类型
        file_handle: 文件句柄
        line: 当前读取的行
        tar_file: tar文件对象（仅TGZ格式使用）
        tar_file_members: tar成员队列（仅TGZ格式使用）

    Example:
        >>> fields = ["id", "name", "value"]
        >>> parser = FileParser("data.csv", fields, ",")
        >>> for row in parser:
        ...     print(row)
        >>> parser.close()

    Note:
        使用时需要捕获IOError异常处理文件读取错误。
    """

    def __init__(
        self,
        file_name: str,
        field_list: List[str],
        field_sep: str,
        file_type: int = FileType.FILE_TYPE_TEXT
    ):
        """
        初始化文件解析器

        Args:
            file_name: 文件路径
            field_list: 字段名列表，用于将每行数据映射为字典
            field_sep: 字段分隔符
            file_type: 文件类型，默认为纯文本
        """
        logger.info(f"FileParser: {file_name}")
        self.file_name: str = file_name
        self.filed_list: List[str] = field_list
        self.filed_sep: str = field_sep
        self.file_handle: Optional[IO[Any]] = None
        self.file_type: int = file_type
        self.line: str = ""
        self.tar_file: Optional[tarfile.TarFile] = None
        self.tar_file_members: queue.Queue[tarfile.TarInfo] = queue.Queue()

    def __iter__(self) -> "FileParser":
        """
        初始化迭代器

        根据文件类型打开相应的文件句柄。

        Returns:
            FileParser实例自身

        Raises:
            IOError: 文件打开失败时抛出
        """
        if self.file_type == FileType.FILE_TYPE_GZIP:
            self.file_handle = gzip.open(self.file_name, "rt", encoding="utf-8")
        elif self.file_type == FileType.FILE_TYPE_ZIP:
            self.file_handle = zipfile.ZipFile(self.file_name, "r")
        elif self.file_type == FileType.FILE_TYPE_BZ2:
            self.file_handle = bz2.open(self.file_name, "rt", encoding="utf-8")
        elif self.file_type == FileType.FILE_TYPE_TGZ:
            self.tar_file = tarfile.open(self.file_name, "r:gz")
            for member in self.tar_file.getmembers():
                if member.isfile():
                    self.tar_file_members.put(member)
        else:
            self.file_handle = open(self.file_name, "r", encoding="utf-8")

        return self

    def __next__(self) -> Dict[str, str]:
        """
        获取下一行数据

        读取文件的下一行，按分隔符拆分后映射为字典。

        Returns:
            字段名到字段值的字典

        Raises:
            StopIteration: 文件读取完毕时抛出
            IOError: 文件读取错误时抛出
        """
        try:
            # TGZ格式特殊处理：需要逐个处理tar包中的文件
            if self.file_type == FileType.FILE_TYPE_TGZ and self.file_handle is None:
                if self.tar_file_members.empty():
                    raise StopIteration
                self.file_handle = self.tar_file.extractfile(self.tar_file_members.get())
                if self.file_handle is None:
                    raise StopIteration

            # 读取一行
            line_bytes = self.file_handle.readline()

            # 处理bytes类型（TGZ格式返回bytes）
            if isinstance(line_bytes, bytes):
                self.line = line_bytes.decode("utf-8")
            else:
                self.line = line_bytes

            # 检查是否到达文件末尾
            if self.line == "":
                self.file_handle.close()
                self.file_handle = None
                # TGZ格式：尝试打开下一个文件
                if self.file_type == FileType.FILE_TYPE_TGZ:
                    return self.__next__()
                raise StopIteration

            # 去除行尾换行符并分割
            if self.line.endswith(os.linesep):
                line_array = self.line[:-len(os.linesep)].split(self.filed_sep)
            elif self.line.endswith("\n"):
                line_array = self.line[:-1].split(self.filed_sep)
            else:
                line_array = self.line.split(self.filed_sep)

            # 将字段列表与值列表组合为字典
            line_dict = dict(zip(self.filed_list, line_array))

        except IOError as error:
            logger.error(f"FileParser read {self.file_name} failed: {error}")
            raise error

        return line_dict

    def close(self) -> None:
        """
        关闭文件句柄

        释放文件资源，包括普通文件句柄和tar文件句柄。
        """
        if self.file_handle:
            self.file_handle.close()
            self.file_handle = None
        if self.tar_file:
            self.tar_file.close()
            self.tar_file = None

    def __enter__(self) -> "FileParser":
        """
        上下文管理器入口

        Returns:
            FileParser实例自身
        """
        return self.__iter__()

    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        """
        上下文管理器退出

        Args:
            exc_type: 异常类型
            exc_val: 异常值
            exc_tb: 异常追溯
        """
        self.close()


if __name__ == "__main__":
    pass



"""
HTML模板工具模块

提供HTML文档生成功能，支持：
- HTML文档结构生成
- 表格生成（支持字典列表和对象列表）
- 样式化邮件模板
- 链接生成
"""
from typing import List, Dict, Any, Union, Optional


class HtmlTemplate:
    """
    HTML模板生成类

    提供静态方法生成HTML文档和组件，主要用于邮件和报表的HTML化。

    Example:
        >>> data = [{"name": "Alice", "age": 30}, {"name": "Bob", "age": 25}]
        >>> html = HtmlTemplate.get_html_table(data)
    """

    # HTML 结构：<html> <head></head> <body></body> </html>
    TEMPLATE_HTML = """ <html> {} {} </html> """

    # HEADER 结构：<head> <meta></meta> <script></script> </head>
    TEMPLATE_HEAD = """ <head><meta charset="utf-8"> {} </head> """

    # BODY
    TEMPLATE_BODY = """
        <body style="height:100%; width: 100%; margin-left:2%;">
        {}
        {}
        </body>
    """

    # CSS样式
    TEMPLATE_CSS = """
        <style>
            .mail-table {
                font-family: Arial, Helvetica, sans-serif;
                border-collapse: collapse;
                min-width: 600px;
            }

            .mail-table td, .mail-table th {
                border: 1px solid #ddd;
                padding: 8px;
            }

            .mail-table tr:nth-child(even){background-color: #f2f2f2;}

            .mail-table tr:hover {background-color: #ddd;}

            .mail-table th {
                padding-top: 6px;
                padding-bottom: 6px;
                text-align: center;
                background-color: #3f98b1;
                color: white;
            }
            ol {
                list-style: none;
                counter-reset: steps;
            }
            ol li {
                counter-increment: steps;
            }
            ol li::before {
                content: counter(steps);
                margin-right: 0.5rem;
                background: #ff6f00;
                color: white;
                width: 1.2em;
                height: 1.2em;
                border-radius: 50%;
                display: inline-grid;
                place-items: center;
                line-height: 1.2em;
            }
            ol ol li::before {
                background: darkorchid;
            }
        </style>
    """

    # TABLE
    TEMPLATE_TABLE = """
         <table class="mail-table">
         <tbody> {} </tbody>
         </table>
    """
    TEMPLATE_TABLE_TR = """ <tr> {} </tr> """
    TEMPLATE_TABLE_TH = """ <th> {} </th> """
    TEMPLATE_TABLE_TD = """ <td> {} </td> """
    TEMPLATE_TABLE_TD_LEFT = """ <td style="text-align:left;"> {} </td> """
    TEMPLATE_TABLE_TD_CENTER = """ <td style="text-align:center;"> {} </td> """

    @staticmethod
    def get_html(content: str) -> str:
        """
        生成完整的HTML文档

        Args:
            content: HTML内容

        Returns:
            完整的HTML文档字符串
        """
        body = HtmlTemplate.TEMPLATE_BODY.format(HtmlTemplate.TEMPLATE_CSS, content)
        html = HtmlTemplate.TEMPLATE_HTML.format("", body)
        return html

    @staticmethod
    def get_html_table(dict_list: List[Union[Dict[str, Any], Any]]) -> str:
        """
        将字典列表转换为HTML表格文档

        Args:
            dict_list: 字典列表或对象列表

        Returns:
            包含表格的完整HTML文档
        """
        content = HtmlTemplate.get_table(dict_list)
        body = HtmlTemplate.TEMPLATE_BODY.format(HtmlTemplate.TEMPLATE_CSS, content)
        html = HtmlTemplate.TEMPLATE_HTML.format("", body)
        return html

    @staticmethod
    def get_table(dict_list: List[Union[Dict[str, Any], Any]]) -> str:
        """
        将字典列表或对象列表转换为HTML表格

        Args:
            dict_list: 字典列表或对象列表

        Returns:
            HTML表格字符串，空列表返回空字符串
        """
        if not dict_list:
            return ""

        # 获取字段列表
        first = dict_list[0]
        if isinstance(first, dict):
            fields = tuple(first.keys())
        else:
            fields = tuple(first.__dict__.keys())

        # 构建表头
        head = ""
        for field in fields:
            head += HtmlTemplate.TEMPLATE_TABLE_TH.format(field)
        head_line = HtmlTemplate.TEMPLATE_TABLE_TR.format(head)
        content = head_line

        # 构建数据行
        for data in dict_list:
            if not isinstance(data, dict):
                data = data.__dict__
            row = ""
            for field in fields:
                value = HtmlTemplate._get_value(data, field)
                row += HtmlTemplate.TEMPLATE_TABLE_TD_CENTER.format(value)
            content += HtmlTemplate.TEMPLATE_TABLE_TR.format(row)

        return HtmlTemplate.TEMPLATE_TABLE.format(content)

    @staticmethod
    def _get_value(data: Dict[str, Any], key: str) -> Any:
        """
        安全获取字典值

        Args:
            data: 数据字典
            key: 键名

        Returns:
            键对应的值，不存在时返回空字符串
        """
        if isinstance(data, dict) and key in data:
            return data[key]
        return ""

    @staticmethod
    def get_tag_a(url: str, desc: str, target: str = "_blank") -> str:
        """
        生成HTML链接标签

        Args:
            url: 链接地址
            desc: 链接文本
            target: 打开方式，默认新窗口打开

        Returns:
            HTML链接标签字符串
        """
        return f'<a href="{url}" target="{target}">{desc}</a>'
